@@ -5,6 +5,7 @@
 #include <stdio.h>
 #include <libguile.h>
 #include <gnome.h>
+#include <guile/gh.h>
 
 #include "../../src/gnumeric.h"
 #include "../../src/symbol.h"
@@ -96,7 +97,7 @@ value_to_scm (Value *val, CellRef cell_ref)
 			return scm_long2num(val->v.v_int);
 
 		case VALUE_FLOAT :
-			return scm_dbl2big(val->v.v_float);
+			return gh_double2scm(val->v.v_float);
 
 		case VALUE_CELLRANGE :
 			return scm_cons(scm_symbolfrom0str("cell-range"),
@@ -104,7 +105,20 @@ value_to_scm (Value *val, CellRef cell_ref)
 						 cell_ref_to_scm(val->v.cell_range.cell_b, cell_ref)));
 
 		case VALUE_ARRAY :
-			return SCM_UNSPECIFIED;
+			{
+				int x, y, i, ii;
+				SCM ls;
+
+				x = val->v.array.x;
+				y = val->v.array.y;
+
+				ls = gh_eval_str("'()");
+
+				for(i = 0; i < y; i++)
+					for(ii = 0; i < x; i++)
+						ls = scm_cons(val->v.array.vals[ii][i], ls);
+				return ls;
+			}
 	}
 
 	return SCM_UNSPECIFIED;
@@ -123,10 +137,14 @@ scm_to_value (SCM scm)
 	}
 	else if (SCM_NFALSEP(scm_number_p(scm)))
 	{
-		if (scm_integer_p(scm))
-			return value_new_int ((int)scm_num2int(scm));
-		else
-			return value_new_float ((float)scm_num2dbl(scm, 0));
+		/* We do not need to do any distinction between an integer or
+		 *  a float here. If we do so, we can crash gnumeric if the
+                 *  size of scm is bigger than the size of int
+		 */
+
+		//return value_new_int((int)scm_num2int(scm));
+
+		return value_new_float ((float_t)scm_num2dbl(scm, 0));
 	}
 	else if (SCM_NIMP(scm) && SCM_CONSP(scm))
 	{
@@ -204,6 +222,11 @@ expr_to_scm (ExprTree *expr, CellRef cell_ref)
 					 expr_to_scm(expr->u.binary.value_a, cell_ref),
 					 expr_to_scm(expr->u.binary.value_b, cell_ref));
 
+		case OPER_PERCENT :
+			return SCM_LIST3(scm_symbolfrom0str("modulo"),
+					 expr_to_scm(expr->u.binary.value_a, cell_ref),
+					 expr_to_scm(expr->u.binary.value_b, cell_ref));
+
 		case OPER_EXP :
 			return SCM_LIST3(scm_symbolfrom0str("expt"),
 					 expr_to_scm(expr->u.binary.value_a, cell_ref),
@@ -232,19 +255,6 @@ expr_to_scm (ExprTree *expr, CellRef cell_ref)
 	return SCM_UNSPECIFIED;
 }
 
-static Value*
-func_scm_eval (FunctionEvalInfo *ei, Value *argv[])
-{
-	SCM result;
-
-	if (argv[0]->type != VALUE_STRING)
-		return value_new_error (&ei->pos, _("Argument must be a Guile expression"));
-
-	result = scm_eval_0str(argv[0]->v.str->str);
-
-	return scm_to_value(result);
-}
-
 static Value*
 func_scm_apply (FunctionEvalInfo *ei, GList *expr_node_list)
 {
@@ -276,12 +286,12 @@ func_scm_apply (FunctionEvalInfo *ei, GList *expr_node_list)
 	{
 		CellRef eval_cell;
 
-		eval_cell.col = ei->pos.eval_col;
-		eval_cell.row = ei->pos.eval_row;
+		eval_cell.col = ei->pos.eval.col;
+		eval_cell.row = ei->pos.eval.row;
 		eval_cell.col_relative = 0;
 		eval_cell.row_relative = 0;
 		eval_cell.sheet = NULL;
-		
+
 		value = eval_expr(ei, (ExprTree*)g_list_nth(expr_node_list, i)->data);
 		if (value == NULL)
 			return value_new_error (&ei->pos, _("Could not evaluate argument"));
@@ -300,7 +310,7 @@ scm_cell_value (SCM scm)
 {
 	CellRef cell_ref = scm_to_cell_ref(scm);
 	Cell *cell;
-	
+
 	g_return_val_if_fail (eval_pos != NULL, SCM_EOL);
 
 	cell = sheet_cell_get(eval_pos->sheet, cell_ref.col, cell_ref.row);
@@ -394,8 +404,10 @@ func_marshal_func (FunctionEvalInfo *ei, Value *argv[])
 	FunctionDefinition *fndef = ei->func_def;
 	SCM args = SCM_EOL, result, function;
 	CellRef dummy = { 0, 0, 0, 0 };
-	int i, count = strlen(fndef->args);
 	EvalPosition *old_eval_pos;
+	int i, min, max;
+
+	function_def_count_args(fndef, &min, &max);
 
 	l = g_list_find_custom(funclist, fndef, (GCompareFunc)fndef_compare);
 	if (l == NULL)
@@ -403,7 +415,7 @@ func_marshal_func (FunctionEvalInfo *ei, Value *argv[])
 
 	function = ((FuncData*)l->data)->function;
 
-	for (i = count - 1; i >= 0; --i)
+	for (i = min - 1; i >= 0; --i)
 		args = scm_cons(value_to_scm(argv[i], dummy), args);
 
 	old_eval_pos = eval_pos;
@@ -451,19 +463,20 @@ no_unloading_for_me (PluginData *pd)
 	return 0;
 }
 
-int init_plugin (PluginData *pd);
-
-int
-init_plugin (PluginData *pd)
+PluginInitResult
+init_plugin (CommandContext *context, PluginData *pd)
 {
 	FunctionCategory *cat;
 	char *init_file_name;
 
+	if (plugin_version_mismatch  (context, pd, GNUMERIC_VERSION))
+		return PLUGIN_QUIET_ERROR;
+
 	/* Initialize just in case. */
 	eval_pos = NULL;
 
 	cat = function_get_category ("Guile");
-	function_add_args  (cat, "scm_eval", "s", "expr",  NULL, func_scm_eval);
+
 	function_add_nodes (cat, "scm_apply", 0, "symbol", NULL, func_scm_apply);
 
 	pd->can_unload = no_unloading_for_me;
@@ -483,5 +496,5 @@ init_plugin (PluginData *pd)
 		  scm_cons(scm_makfrom0str(init_file_name), SCM_EOL),
 		  SCM_EOL);
 
-	return 0;
+	return PLUGIN_OK;
 }
