@@ -42,16 +42,23 @@ typedef struct {
 	Sheet *sheet;
 } SheetServant;
 
-static void
+static inline void
 out_of_range (CORBA_Environment *ev)
 {
 	CORBA_exception_set (ev, CORBA_USER_EXCEPTION, ex_GNOME_Gnumeric_Sheet_OutOfRange, NULL);
 }
 
+CommandContext *
+command_context_corba (void)
+{
+	/* FIXME FIXME FIXME : corba context */
+	return NULL;
+}
+
 /*
  * Parses a list of ranges, returns a GList containing pointers to
  * Value structures.  Sets the return_list pointer to point to a
- * a listof those values.
+ * a list of those values.
  *
  * Returns TRUE on successfully parsing the string, FALSE otherwise
  */
@@ -61,7 +68,7 @@ corba_range_parse (Sheet *sheet, const char *range_spec, GSList **return_list)
 	GSList *list;
 	
 	list = range_list_parse (sheet, range_spec);
-	if (list){
+	if (list) {
 		*return_list = list;
 		return TRUE;
 	} else {
@@ -177,7 +184,7 @@ Sheet_selection_copy (PortableServer_Servant servant, CORBA_Environment *ev)
 {
 	Sheet *sheet = sheet_from_servant (servant);
 
-	sheet_selection_copy (sheet);
+	sheet_selection_copy (command_context_corba (), sheet);
 }
 
 static void
@@ -185,7 +192,7 @@ Sheet_selection_cut (PortableServer_Servant servant, CORBA_Environment *ev)
 {
 	Sheet *sheet = sheet_from_servant (servant);
 
-	sheet_selection_cut (sheet);
+	sheet_selection_cut (command_context_corba (), sheet);
 }
 
 static void
@@ -198,7 +205,8 @@ Sheet_selection_paste (PortableServer_Servant servant,
 	verify_col (dest_col);
 	verify_row (dest_row);
 	
-	sheet_selection_paste (sheet, dest_col, dest_row, paste_flags, 0);
+	sheet_selection_paste (command_context_corba (), sheet,
+			       dest_col, dest_row, paste_flags, 0);
 }
 
 static void
@@ -211,7 +219,8 @@ Sheet_clear_region (PortableServer_Servant servant,
 
 	verify_region (start_col, start_row, end_col, end_row);
 
-	sheet_clear_region (sheet, start_col, start_row, end_col, end_row, NULL);
+	sheet_clear_region (command_context_corba (), sheet, start_col, start_row,
+			    end_col, end_row, TRUE);
 }
 
 static void
@@ -223,7 +232,8 @@ Sheet_clear_region_content (PortableServer_Servant servant,
 	Sheet *sheet = sheet_from_servant (servant);
 
 	verify_region (start_col, start_row, end_col, end_row);
-	sheet_clear_region_content (sheet, start_col, start_row, end_col, end_row, NULL);
+	sheet_clear_region_content (command_context_corba (), sheet,
+				    start_col, start_row, end_col, end_row);
 }
 
 static void
@@ -424,13 +434,13 @@ Sheet_cell_get_text (PortableServer_Servant servant,
 		     CORBA_Environment *ev)
 {
 	Sheet *sheet = sheet_from_servant (servant);
-	Cell *cell;
+	Cell  *cell;
 
 	verify_col_val (col, NULL);
 	verify_row_val (row, NULL);
 	
 	cell = sheet_cell_get (sheet, col, row);
-	if (cell){
+	if (cell) {
 		char *str;
 
 		str = cell_get_text (cell);
@@ -795,7 +805,8 @@ Sheet_insert_col (PortableServer_Servant servant,
 	Sheet *sheet = sheet_from_servant (servant);
 
 	verify_col (col);
-	sheet_insert_col (sheet, col, count);
+	sheet_insert_cols (command_context_corba (), sheet,
+			   col, count);
 }
 
 static void
@@ -806,7 +817,8 @@ Sheet_delete_col (PortableServer_Servant servant,
 	Sheet *sheet = sheet_from_servant (servant);
 
 	verify_col (col);
-	sheet_delete_col (sheet, col, count);
+	sheet_delete_cols (command_context_corba (), sheet,
+			   col, count);
 }
 
 static void
@@ -817,7 +829,8 @@ Sheet_insert_row (PortableServer_Servant servant,
 	Sheet *sheet = sheet_from_servant (servant);
 
 	verify_row (row);
-	sheet_insert_row (sheet, row, count);
+	sheet_insert_rows (command_context_corba (), sheet,
+			   row, count);
 }
 
 static void
@@ -828,7 +841,8 @@ Sheet_delete_row (PortableServer_Servant servant,
 	Sheet *sheet = sheet_from_servant (servant);
 
 	verify_row (row);
-	sheet_delete_row (sheet, row, count);
+	sheet_delete_rows (command_context_corba (), sheet,
+			   row, count);
 }
 
 static void
@@ -843,7 +857,8 @@ Sheet_shift_rows (PortableServer_Servant servant,
 	verify_row (end_row);
 	verify_col (col);
 
-	sheet_shift_rows (sheet, col, start_row, end_row, count);
+	sheet_shift_rows (command_context_corba (), sheet,
+			  col, start_row, end_row, count);
 }
 
 static void
@@ -858,7 +873,8 @@ Sheet_shift_cols (PortableServer_Servant servant,
 	verify_row (start_row);
 	verify_row (end_row);
 
-	sheet_shift_cols (sheet, col, start_row, end_row, count);
+	sheet_shift_cols (command_context_corba (), sheet,
+			  col, start_row, end_row, count);
 }
 
 static GNOME_Gnumeric_Sheet_ValueVector *
@@ -900,7 +916,7 @@ Sheet_range_get_values (PortableServer_Servant servant, const CORBA_char *range,
 		vector->_maximum = 0;
 
 		return vector;
-}
+	}
 
 	/*
 	 * Fill in the vector
@@ -989,20 +1005,149 @@ Sheet_range_set_format (PortableServer_Servant servant,
 	range_list_destroy (ranges);
 }
 
+static void
+Sheet_range_set_font (PortableServer_Servant servant,
+		      const CORBA_char *range,
+		      const CORBA_char *font,
+		      CORBA_short points,
+		      CORBA_Environment *ev)
+{
+	Sheet *sheet = sheet_from_servant (servant);
+	GSList *ranges;
+	MStyle *mstyle;
+
+	verify_range (sheet, range, &ranges);
+
+	cell_freeze_redraws ();
+
+	mstyle = mstyle_new ();
+	mstyle_set_font_name (mstyle, font);
+	mstyle_set_font_size (mstyle, points);
+	ranges_set_style (sheet, ranges, mstyle);
+
+	cell_thaw_redraws ();
+	
+	range_list_destroy (ranges);
+}
+
+static void
+Sheet_range_set_foreground (PortableServer_Servant servant,
+			    const CORBA_char *range,
+			    const CORBA_char *color,
+			    CORBA_Environment *ev)
+{
+	Sheet   *sheet = sheet_from_servant (servant);
+	GSList  *ranges;
+	MStyle  *mstyle;
+	GdkColor c;
+
+	verify_range (sheet, range, &ranges);
+	
+	gdk_color_parse (color, &c);
+	cell_freeze_redraws ();
+
+	mstyle = mstyle_new ();
+	mstyle_set_color (mstyle, MSTYLE_COLOR_FORE,
+			  style_color_new (c.red, c.green, c.blue));
+	ranges_set_style (sheet, ranges, mstyle);
+
+	cell_thaw_redraws ();
+	
+	range_list_destroy (ranges);
+}
+
+static void
+Sheet_range_set_background (PortableServer_Servant servant,
+			    const CORBA_char *range,
+			    const CORBA_char *color,
+			    CORBA_Environment *ev)
+{
+	Sheet   *sheet = sheet_from_servant (servant);
+	GSList  *ranges;
+	MStyle  *mstyle;
+	GdkColor c;
+
+	verify_range (sheet, range, &ranges);
+	
+	gdk_color_parse (color, &c);
+	cell_freeze_redraws ();
+
+	mstyle = mstyle_new ();
+	mstyle_set_color (mstyle, MSTYLE_COLOR_BACK,
+			  style_color_new (c.red, c.green, c.blue));
+	ranges_set_style (sheet, ranges, mstyle);
+
+	cell_thaw_redraws ();
+	
+	range_list_destroy (ranges);
+}
+
+static void
+Sheet_range_set_pattern (PortableServer_Servant servant,
+			 const CORBA_char      *range,
+			 CORBA_long             pattern,
+			 CORBA_Environment     *ev)
+{
+	Sheet   *sheet = sheet_from_servant (servant);
+	GSList  *ranges;
+	MStyle  *mstyle;
+
+	verify_range (sheet, range, &ranges);
+	
+	cell_freeze_redraws ();
+
+	mstyle = mstyle_new ();
+	mstyle_set_pattern (mstyle, pattern);
+	ranges_set_style (sheet, ranges, mstyle);
+
+	cell_thaw_redraws ();
+	
+	range_list_destroy (ranges);
+}
+
+static void
+Sheet_range_set_alignment (PortableServer_Servant servant,
+			   const CORBA_char      *range,
+			   CORBA_long             halign,
+			   CORBA_long             valign,
+			   CORBA_long             orientation,
+			   CORBA_boolean          auto_return,
+			   CORBA_Environment     *ev)
+{
+	Sheet   *sheet = sheet_from_servant (servant);
+	GSList  *ranges;
+	MStyle  *mstyle;
+
+	verify_range (sheet, range, &ranges);
+	
+	mstyle = mstyle_new ();
+	mstyle_set_align_h (mstyle, halign);
+	mstyle_set_align_v (mstyle, valign);
+	mstyle_set_orientation (mstyle, orientation);
+	mstyle_set_fit_in_cell (mstyle, (gboolean) auto_return);
+	ranges_set_style (sheet, ranges, mstyle);
+
+	cell_thaw_redraws ();
+	
+	range_list_destroy (ranges);
+}
+
 static CORBA_long
 Sheet_max_cols_used (PortableServer_Servant servant, CORBA_Environment *ev)
 {
-	Sheet *sheet = sheet_from_servant (servant);
+	Sheet *sheet  = sheet_from_servant (servant);
+	Range  extent = sheet_get_extent (sheet);
 
-	return sheet->cols.max_used;
+	return extent.end.col;
 }
 
 static CORBA_long
 Sheet_max_rows_used (PortableServer_Servant servant, CORBA_Environment *ev)
 {
 	Sheet *sheet = sheet_from_servant (servant);
+	Range  extent = sheet_get_extent (sheet);
 
-	return sheet->rows.max_used;
+	return extent.end.row;
 }
 
 static CORBA_double
@@ -1093,10 +1238,15 @@ Sheet_corba_class_init (void)
 	/*
 	 * Region based routines
 	 */
-	gnome_gnumeric_sheet_epv.range_get_values = Sheet_range_get_values;
-	gnome_gnumeric_sheet_epv.range_set_text = Sheet_range_set_text;
+	gnome_gnumeric_sheet_epv.range_get_values  = Sheet_range_get_values;
+	gnome_gnumeric_sheet_epv.range_set_text    = Sheet_range_set_text;
 	gnome_gnumeric_sheet_epv.range_set_formula = Sheet_range_set_formula;
-	gnome_gnumeric_sheet_epv.range_set_format = Sheet_range_set_format;
+	gnome_gnumeric_sheet_epv.range_set_format  = Sheet_range_set_format;
+	gnome_gnumeric_sheet_epv.range_set_font    = Sheet_range_set_font;
+	gnome_gnumeric_sheet_epv.range_set_foreground = Sheet_range_set_foreground;
+	gnome_gnumeric_sheet_epv.range_set_background = Sheet_range_set_background;
+	gnome_gnumeric_sheet_epv.range_set_pattern    = Sheet_range_set_pattern;
+	gnome_gnumeric_sheet_epv.range_set_alignment  = Sheet_range_set_alignment;
 
 	gnome_gnumeric_sheet_epv.set_dirty = Sheet_set_dirty;
 
