@@ -17,8 +17,6 @@
 #include "utils.h"
 #include "ranges.h"
 
-static Value *eval_expr_real (FunctionEvalInfo *s, ExprTree const *tree);
-
 EvalPosition *
 eval_pos_init (EvalPosition *eval_pos, Sheet *sheet, int col, int row)
 {
@@ -27,8 +25,8 @@ eval_pos_init (EvalPosition *eval_pos, Sheet *sheet, int col, int row)
 	g_return_val_if_fail (IS_SHEET (sheet), NULL);
 
 	eval_pos->sheet = sheet;
-	eval_pos->eval_col = col;
-	eval_pos->eval_row = row;
+	eval_pos->eval.col = col;
+	eval_pos->eval.row = row;
 
 	return eval_pos;
 }
@@ -108,8 +106,8 @@ func_eval_info_pos (FunctionEvalInfo *eval_info, const EvalPosition *eval_pos)
 	return func_eval_info_init (
 		eval_info,
 		eval_pos->sheet,
-		eval_pos->eval_col,
-		eval_pos->eval_row);
+		eval_pos->eval.col,
+		eval_pos->eval.row);
 }
 
 ExprTree *
@@ -328,8 +326,8 @@ expr_tree_array_formula_corner (ExprTree const *expr, EvalPosition const *pos)
 		g_return_val_if_fail (pos->sheet != NULL, NULL);
 
 		corner = sheet_cell_get (pos->sheet,
-					 pos->eval_col - expr->u.array.x,
-					 pos->eval_row - expr->u.array.y);
+					 pos->eval.col - expr->u.array.x,
+					 pos->eval.row - expr->u.array.y);
 		((ExprTree *)expr)->u.array.corner.cell = corner;
 	}
 
@@ -423,185 +421,75 @@ expr_tree_unref (ExprTree *tree)
 	do_expr_tree_unref (tree);
 }
 
-static void
-cell_ref_make_absolute (CellRef *cell_ref, int eval_col, int eval_row)
-{
-	g_return_if_fail (cell_ref != NULL);
-
-	if (cell_ref->col_relative)
-		cell_ref->col = eval_col + cell_ref->col;
-
-	if (cell_ref->row_relative)
-		cell_ref->row = eval_row + cell_ref->row;
-
-	cell_ref->row_relative = 0;
-	cell_ref->col_relative = 0;
-}
-
-static void
-free_values (Value **values, int top)
-{
-	int i;
-
-	for (i = 0; i < top; i++)
-		if (values[i])
-			value_release (values [i]);
-	g_free (values);
-}
-
 static Value *
-eval_funcall (FunctionEvalInfo *s, ExprTree const *tree)
+eval_funcall (FunctionEvalInfo *ei, ExprTree const *tree)
 {
 	const Symbol *sym;
 	FunctionDefinition *fd;
-	GList *l;
-	int argc, arg;
-	Value *v = NULL;
-	Sheet *sheet;
-	int eval_col;
-	int eval_row;
+	GList *args;
 
-	g_return_val_if_fail (s, NULL);
-	g_return_val_if_fail (tree, NULL);
+	g_return_val_if_fail (ei != NULL, NULL);
+	g_return_val_if_fail (tree != NULL, NULL);
 
 	sym = tree->u.function.symbol;
 
-	sheet    = s->pos.sheet;
-	eval_col = s->pos.eval_col;
-	eval_row = s->pos.eval_row;
-
-	l = tree->u.function.arg_list;
-	argc = g_list_length (l);
-
 	if (sym->type != SYMBOL_FUNCTION)
-		return value_new_error (&s->pos, _("Internal error"));
+		return value_new_error (&ei->pos, _("Internal error"));
 
 	fd = (FunctionDefinition *)sym->data;
-	s->func_def = fd;
+	ei->func_def = fd;
+	args = tree->u.function.arg_list;
 
-	if (fd->fn_type == FUNCTION_NODES) {
-		/* Functions that deal with ExprNodes */		
-		v = fd->fn.fn_nodes (s, l);
-	} else {
-		/* Functions that take pre-computed Values */
-		Value **values;
-		int fn_argc_min = 0, fn_argc_max = 0, var_len = 0;
-		const char *arg_type = fd->args;
-		const char *argptr = fd->args;
+	return function_call_with_list (ei, args);
+}
 
-		/* Get variable limits */
-		while (*argptr){
-			if (*argptr++ == '|'){
-				var_len = 1;
-				continue;
-			}
-			if (!var_len)
-				fn_argc_min++;
-			fn_argc_max++;
+/**
+ * expr_implicit_intersection :
+ * @ei: EvalInfo containing valid fd!
+ * @v: a VALUE_CELLRANGE
+ * 
+ * Attempt to find the intersection between the calling cell and
+ * some element of the 1 one unit wide or tall range.
+ *
+ * Always release the value passed in.
+ *
+ * Return value: 
+ *     If the intersection succeeded return a duplicate of the value
+ *     at the intersection point.  This value needs to be freed.
+ **/
+Value *
+expr_implicit_intersection (EvalPosition const * const pos,
+			    Value * const v)
+{
+	/*
+	 * Handle the implicit union of a single row or
+	 * column with the eval position.
+	 * NOTE : We do not need to know if this is expression is
+	 * being evaluated as an array or not because we can differentiate
+	 * based on the required type for the argument.
+	 */
+	Value *res = NULL;
+	CellRef const * const a = & v->v.cell_range.cell_a;
+	CellRef const * const b = & v->v.cell_range.cell_b;
+
+	if (a->sheet == b->sheet) {
+		int a_col, a_row, b_col, b_row;
+		cell_get_abs_col_row (a, &pos->eval, &a_col, &a_row);
+		cell_get_abs_col_row (b, &pos->eval, &b_col, &b_row);
+		if (a_row == b_row) {
+			int const c = pos->eval.col;
+			if (a_col <= c && c <= b_col)
+				res = value_duplicate (value_area_get_x_y (pos, v, c - a_col, 0));
 		}
 
-		if (argc > fn_argc_max || argc < fn_argc_min)
-			return value_new_error (&s->pos, _("Invalid number of arguments"));
-
-		values = g_new (Value *, fn_argc_max);
-
-		for (arg = 0; l; l = l->next, arg++, arg_type++){
-			ExprTree *t = (ExprTree *) l->data;
-			gboolean type_mismatch = FALSE;
-			Value *v;
-
-			if (*arg_type=='|')
-				arg_type++;
-
-			if ((*arg_type != 'A' &&          /* This is so a cell reference */
-			     *arg_type != 'r') ||         /* can be converted to a cell range */
-			    !t || (t->oper != OPER_VAR)) { /* without being evaluated */
-				if ((v = eval_expr_real (s, t)) == NULL)
-					goto free_list;
-			} else {
-				g_assert (t->oper == OPER_VAR);
-				v = value_new_cellrange (&t->u.ref,
-							 &t->u.ref);
-			}
-
-			switch (*arg_type){
-			case 'f':
-			case 'b':
-				/*
-				 * Handle the implicit union of a single row or
-				 * column with the eval position
-				 */
-				if (v->type == VALUE_CELLRANGE) {
-					CellRef a = v->v.cell_range.cell_a;
-					CellRef b = v->v.cell_range.cell_b;
-					cell_ref_make_absolute (&a, eval_col,
-								eval_row);
-					cell_ref_make_absolute (&b, eval_col,
-								eval_row);
-
-					if (a.sheet !=  b.sheet)
-						type_mismatch = TRUE;
-					else if (a.row == b.row) {
-						int const c = s->pos.eval_col;
-						if (a.col <= c && c <= b.col)
-							v = value_duplicate (value_area_get_x_y (&s->pos, v, c - a.col, 0));
-						else
-							type_mismatch = TRUE;
-					} else if (a.col == b.col) {
-						int const r = s->pos.eval_row;
-						if (a.row <= r && r <= b.row)
-							v = value_duplicate (value_area_get_x_y (&s->pos, v, 0, r - a.row));
-						else
-							type_mismatch = TRUE;
-					} else
-						type_mismatch = TRUE;
-				} else if (v->type != VALUE_INTEGER &&
-					 v->type != VALUE_FLOAT &&
-					 v->type != VALUE_BOOLEAN)
-					type_mismatch = TRUE;
-				break;
-			case 's':
-				if (v->type != VALUE_STRING)
-					type_mismatch = TRUE;
-				break;
-			case 'r':
-				if (v->type != VALUE_CELLRANGE)
-					type_mismatch = TRUE;
-				else {
-					cell_ref_make_absolute (&v->v.cell_range.cell_a, eval_col, eval_row);
-					cell_ref_make_absolute (&v->v.cell_range.cell_b, eval_col, eval_row);
-				}
-				break;
-			case 'a':
-				if (v->type != VALUE_ARRAY)
-					type_mismatch = TRUE;
-				break;
-			case 'A':
-				if (v->type != VALUE_ARRAY &&
-				    v->type != VALUE_CELLRANGE)
-					type_mismatch = TRUE;
-
-				if (v->type == VALUE_CELLRANGE) {
-					cell_ref_make_absolute (&v->v.cell_range.cell_a, eval_col, eval_row);
-					cell_ref_make_absolute (&v->v.cell_range.cell_b, eval_col, eval_row);
-				}
-				break;
-			}
-			values [arg] = v;
-			if (type_mismatch){
-				free_values (values, arg + 1);
-				return value_new_error (&s->pos,
-							gnumeric_err_VALUE);
-			}
+		if (a_col == b_col) {
+			int const r = pos->eval.row;
+			if (a_row <= r && r <= b_row)
+				res = value_duplicate (value_area_get_x_y (pos, v, 0, r - a_row));
 		}
-		while (arg < fn_argc_max)
-			values [arg++] = NULL;
-		v = fd->fn.fn_args (s, values);
-
-	free_list:
-		free_values (values, arg);
 	}
-	return v;
+	value_release (v);
+	return res;
 }
 
 typedef enum {
@@ -650,9 +538,12 @@ compare_float_float (Value const * const va, Value const * const vb)
 
 /*
  * Compares two (Value *) and returns one of compare_t
+ *
+ * if pos is non null it will perform implict intersection for
+ * cellranges.
  */
 static compare_t
-compare (const Value *a, const Value *b)
+compare (Value const * const a, Value const * const b)
 {
 	ValueType ta, tb;
 
@@ -686,10 +577,6 @@ compare (const Value *a, const Value *b)
 				return IS_LESS;
 		}
 		default :
-			/*
-			 * TODO : Add implicit intersection here if not in
-			 * array mode
-			 * */
 			return TYPE_MISMATCH;
 		}
 	} else if (tb == VALUE_STRING) {
@@ -703,10 +590,6 @@ compare (const Value *a, const Value *b)
 			return IS_GREATER;
 
 		default :
-			/*
-			 * TODO : Add implicit intersection here if not in
-			 * array mode
-			 * */
 			return TYPE_MISMATCH;
 		}
 	}
@@ -749,12 +632,8 @@ eval_range (FunctionEvalInfo *s, Value *v)
 	int r, c;
 	int const gen = s->pos.sheet->workbook->generation;
 
-	cell_get_abs_col_row (a,
-			      s->pos.eval_col, s->pos.eval_row,
-			      &start_col, &start_row);
-	cell_get_abs_col_row (b,
-			      s->pos.eval_col, s->pos.eval_row,
-			      &end_col, &end_row);
+	cell_get_abs_col_row (a, &s->pos.eval, &start_col, &start_row);
+	cell_get_abs_col_row (b, &s->pos.eval, &end_col, &end_row);
 
 	if (a->sheet != b->sheet) {
 		g_warning ("3D references not-fully supported.\n"
@@ -772,18 +651,13 @@ eval_range (FunctionEvalInfo *s, Value *v)
 }
 
 static Value *
-eval_expr_real (FunctionEvalInfo *s, ExprTree const *tree)
+eval_expr_real (FunctionEvalInfo * const s, ExprTree const * const tree)
 {
 	Value *res = NULL, *a = NULL, *b = NULL;
 	
 	g_return_val_if_fail (tree != NULL, NULL);
 	g_return_val_if_fail (s != NULL, NULL);
 
-	/* FIXME FIXME : We need to rework support for
-	 * ranges and builtin operators to support the
-	 * implicit intersection rule AND figure out
-	 * how to turn it off when evaluating as an array
-	 */
 	switch (tree->oper){
 	case OPER_EQUAL:
 	case OPER_NOT_EQUAL:
@@ -794,14 +668,30 @@ eval_expr_real (FunctionEvalInfo *s, ExprTree const *tree)
 		int comp;
 
 		a = eval_expr_real (s, tree->u.binary.value_a);
-		if (a != NULL && a->type == VALUE_ERROR)
-			return a;
+		if (a != NULL) {
+			if (a->type == VALUE_CELLRANGE) {
+				a = expr_implicit_intersection (&s->pos, a);
+				if (a == NULL)
+					return value_new_error (&s->pos, gnumeric_err_VALUE);
+			} else if (a->type == VALUE_ERROR)
+				return a;
+		}
 
 		b = eval_expr_real (s, tree->u.binary.value_b);
-		if (b != NULL && b->type == VALUE_ERROR) {
-			if (a != NULL)
-				value_release (a);
-			return b;
+		if (b != NULL) {
+			Value *res = NULL;
+			if (b->type == VALUE_CELLRANGE) {
+				b = expr_implicit_intersection (&s->pos, b);
+				if (b == NULL)
+					res = value_new_error (&s->pos, gnumeric_err_VALUE);
+			} else if (b->type == VALUE_ERROR)
+				res = b;
+
+			if (res != NULL) {
+				if (a != NULL)
+					value_release (a);
+				return res;
+			}
 		}
 
 		comp = compare (a, b);
@@ -811,7 +701,7 @@ eval_expr_real (FunctionEvalInfo *s, ExprTree const *tree)
 		if (b != NULL)
 			value_release (b);
 
-		if (comp == TYPE_MISMATCH){
+		if (comp == TYPE_MISMATCH) {
 			/* TODO TODO TODO : Make error more informative
 			 *    regarding what is comparing to what
 			 */
@@ -824,7 +714,7 @@ eval_expr_real (FunctionEvalInfo *s, ExprTree const *tree)
 			return value_new_error (&s->pos, gnumeric_err_VALUE);
 		}
 
-		switch (tree->oper){
+		switch (tree->oper) {
 		case OPER_EQUAL:
 			res = value_new_bool (comp == IS_EQUAL);
 			break;
@@ -862,9 +752,6 @@ eval_expr_real (FunctionEvalInfo *s, ExprTree const *tree)
 	case OPER_MULT:
 	case OPER_DIV:
 	case OPER_EXP:
-	        /* Garantees that a != NULL */
-		a = eval_expr (s, tree->u.binary.value_a);
-
 		/*
 		 * Priority
 		 * 1) Error from A
@@ -874,6 +761,16 @@ eval_expr_real (FunctionEvalInfo *s, ExprTree const *tree)
 		 * 5) result of operation, or error specific to the operation
 		 */
 
+	        /* Garantees that a != NULL */
+		a = eval_expr (s, tree->u.binary.value_a);
+
+		/* Handle implicit intersection */
+		if (a->type == VALUE_CELLRANGE) {
+			a = expr_implicit_intersection (&s->pos, a);
+			if (a == NULL)
+				return value_new_error (&s->pos, gnumeric_err_VALUE);
+		}
+
 		/* 1) Error from A */
 		if (a->type == VALUE_ERROR)
 			return a;
@@ -887,6 +784,13 @@ eval_expr_real (FunctionEvalInfo *s, ExprTree const *tree)
 	        /* Garantees that b != NULL */
 		b = eval_expr (s, tree->u.binary.value_b);
 
+		/* Handle implicit intersection */
+		if (b->type == VALUE_CELLRANGE) {
+			b = expr_implicit_intersection (&s->pos, a);
+			if (b == NULL)
+				return value_new_error (&s->pos, gnumeric_err_VALUE);
+		}
+
 		/* 3) Error from B */
 		if (b->type == VALUE_ERROR) {
 			value_release (a);
@@ -996,8 +900,17 @@ eval_expr_real (FunctionEvalInfo *s, ExprTree const *tree)
 	case OPER_NEG:
 	        /* Garantees that a != NULL */
 		a = eval_expr (s, tree->u.value);
+
+		/* Handle implicit intersection */
+		if (a->type == VALUE_CELLRANGE) {
+			a = expr_implicit_intersection (&s->pos, a);
+			if (a == NULL)
+				return value_new_error (&s->pos, gnumeric_err_VALUE);
+		}
+
 		if (a->type == VALUE_ERROR)
 			return a;
+
 		if (!VALUE_IS_NUMBER (a)){
 			value_release (a);
 			return value_new_error (&s->pos, gnumeric_err_VALUE);
@@ -1061,7 +974,7 @@ eval_expr_real (FunctionEvalInfo *s, ExprTree const *tree)
 		}
 
 		ref = &tree->u.ref;
-		cell_get_abs_col_row (ref, s->pos.eval_col, s->pos.eval_row, &col, &row);
+		cell_get_abs_col_row (ref, &s->pos.eval, &col, &row);
 
 		cell_sheet = eval_sheet (ref->sheet, s->pos.sheet);
 		cell = sheet_cell_get (cell_sheet, col, row);
@@ -1083,8 +996,8 @@ eval_expr_real (FunctionEvalInfo *s, ExprTree const *tree)
 	case OPER_ARRAY:
 	{
 		/* The upper left corner manages the recalc of the expr */
-		int const x = tree->u.array.x;
-		int const y = tree->u.array.y;
+		int x = tree->u.array.x;
+		int y = tree->u.array.y;
 		if (x == 0 && y == 0){
 			/* Release old value if necessary */
 			a = tree->u.array.corner.func.value;
@@ -1092,6 +1005,21 @@ eval_expr_real (FunctionEvalInfo *s, ExprTree const *tree)
 				value_release (a);
 
 			/* Store real result (cast away const)*/
+			/* FIXME : Call a wrapper routine that will iterate over the
+			 * the array and evaluate the expression for all elements
+			 *
+			 * Figure out when to iterate and when to do array operations.
+			 * ie
+			 * A1:A3 = '=B1:B3^2'  Will iterate over all the elements and
+			 *     re-evaluate.
+			 * whereas
+			 * A1:A3 = '=bob(B1:B3)'  Will call bob once if it returns an
+			 *     array.
+			 *
+			 * This may be as simple as evaluating the corner.  If that is
+			 * is an array return the result, else build an array and
+			 * iterate over the elements, but that theory needs validation.
+			 */
 			a = eval_expr_real (s, tree->u.array.corner.func.expr);
 			*((Value **)&(tree->u.array.corner.func.value)) = a;
 		} else {
@@ -1108,17 +1036,21 @@ eval_expr_real (FunctionEvalInfo *s, ExprTree const *tree)
 			int const num_x = value_area_get_width (&s->pos, a);
 			int const num_y = value_area_get_height (&s->pos, a);
 
-			if (x < num_x && y < num_y){
-				/* Evaluate relative to the upper left corner */
-				EvalPosition tmp_ep = s->pos;
-				tmp_ep.eval_col -= x;
-				tmp_ep.eval_row -= y;
-				a = (Value *)value_area_get_x_y (
-					&tmp_ep, a, x, y);
-			} else
+			/* Evaluate relative to the upper left corner */
+			EvalPosition tmp_ep = s->pos;
+			tmp_ep.eval.col -= x;
+			tmp_ep.eval.row -= y;
+
+			/* If the src array is 1 element wide or tall we wrap */
+			if (x >= 1 && num_x == 1)
+				x = 0;
+			if (y >= 1 && num_y == 1)
+				y = 0;
+			if (x >= num_x || y >= num_y)
 				return value_new_error (&s->pos, gnumeric_err_NA);
-		} else if (x >= 1 || y >= 1)
-			return value_new_error (&s->pos, gnumeric_err_NA);
+
+			a = (Value *)value_area_get_x_y (&tmp_ep, a, x, y);
+		}
 
 		if (a == NULL)
 			return NULL;
@@ -1144,41 +1076,43 @@ eval_expr (FunctionEvalInfo *s, ExprTree const *tree)
 }
 
 int
-cell_ref_get_abs_col (CellRef const *ref, EvalPosition const *pos)
+cell_ref_get_abs_col (CellRef const * const ref, EvalPosition const * const pos)
 {
 	g_return_val_if_fail (ref != NULL, 0);
 	g_return_val_if_fail (pos != NULL, 0);
 
 	if (ref->col_relative)
-		return pos->eval_col + ref->col;
+		return pos->eval.col + ref->col;
 	return ref->col;
 
 }
 
 int
-cell_ref_get_abs_row (CellRef const *ref, EvalPosition const *pos)
+cell_ref_get_abs_row (CellRef const * const ref, EvalPosition const * const pos)
 {
 	g_return_val_if_fail (ref != NULL, 0);
 	g_return_val_if_fail (pos != NULL, 0);
 
 	if (ref->row_relative)
-		return pos->eval_row + ref->row;
+		return pos->eval.row + ref->row;
 	return ref->row;
 }
 
 
 void
-cell_get_abs_col_row (const CellRef *cell_ref, int eval_col, int eval_row, int *col, int *row)
+cell_get_abs_col_row (CellRef const * const cell_ref,
+		      CellPos const * const pos,
+		      int * const col, int * const row)
 {
 	g_return_if_fail (cell_ref != NULL);
 
 	if (cell_ref->col_relative)
-		*col = eval_col + cell_ref->col;
+		*col = pos->col + cell_ref->col;
 	else
 		*col = cell_ref->col;
 
 	if (cell_ref->row_relative)
-		*row = eval_row + cell_ref->row;
+		*row = pos->row + cell_ref->row;
 	else
 		*row = cell_ref->row;
 }
@@ -1225,7 +1159,7 @@ do_expr_decode_tree (ExprTree *tree, ParsePosition const *pp,
 
 	op = tree->oper;
 
-	switch (op){
+	switch (op) {
 	case OPER_ANY_BINARY: {
 		char *a, *b, *res;
 		char const *opname;
@@ -1282,12 +1216,12 @@ do_expr_decode_tree (ExprTree *tree, ParsePosition const *pp,
 		arg_list = tree->u.function.arg_list;
 		argc = g_list_length (arg_list);
 
-		if (argc){
+		if (argc) {
 			int i, len = 0;
 			args = g_malloc (sizeof (char *) * argc);
 
 			i = 0;
-			for (l = arg_list; l; l = l->next, i++){
+			for (l = arg_list; l; l = l->next, i++) {
 				ExprTree *t = l->data;
 
 				args [i] = do_expr_decode_tree (t, pp, 0);
@@ -1298,13 +1232,14 @@ do_expr_decode_tree (ExprTree *tree, ParsePosition const *pp,
 
 			i = 0;
 			sum [0] = 0;
-			for (l = arg_list; l; l = l->next, i++){
+			for (l = arg_list; l; l = l->next, i++) {
 				strcat (sum, args [i]);
 				if (l->next)
 					strcat (sum, ",");
 			}
 
-			res = g_strconcat (fd->name, "(", sum, ")", NULL);
+			res = g_strconcat (function_def_get_name (fd),
+					   "(", sum, ")", NULL);
 			g_free (sum);
 
 			for (i = 0; i < argc; i++)
@@ -1313,7 +1248,8 @@ do_expr_decode_tree (ExprTree *tree, ParsePosition const *pp,
 
 			return res;
 		} else
-			return g_strconcat (fd->name, "()", NULL);
+			return g_strconcat (function_def_get_name (fd),
+					    "()", NULL);
 	}
 
 	case OPER_NAME:
@@ -1470,7 +1406,7 @@ cellref_relocate (CellRef * const ref,
 		range_contains (&rinfo->origin, col, row);
 	gboolean const from_inside =
 		(rinfo->origin_sheet == pos->sheet) &&
-		range_contains (&rinfo->origin, pos->eval_col, pos->eval_row);
+		range_contains (&rinfo->origin, pos->eval.col, pos->eval.row);
 
 	/* Case (a) */
 	if (!from_inside && !to_inside)
@@ -1511,9 +1447,9 @@ cellref_relocate (CellRef * const ref,
 	    return CELLREF_RELOCATE_ERR;
 
 	if (ref->col_relative)
-		col -= pos->eval_col;
+		col -= pos->eval.col;
 	if (ref->row_relative)
-		row -= pos->eval_row;
+		row -= pos->eval.row;
 
 	if (ref->sheet != ref_sheet || ref->col != col || ref->row != row) {
 		ref->sheet = ref_sheet;
