@@ -1,12 +1,7 @@
 #ifndef GNUMERIC_EXPR_H
 #define GNUMERIC_EXPR_H
 
-/* Forward references for structures.  */
-typedef struct _ExprTree		ExprTree;
-typedef struct _ArrayRef		ArrayRef;
-typedef struct _ExprName		ExprName;
-typedef struct _FunctionEvalInfo	FunctionEvalInfo;
-
+#include "gnumeric.h"
 #include "value.h"
 #include "sheet.h"
 #include "symbol.h"
@@ -131,31 +126,6 @@ FunctionEvalInfo *func_eval_info_init (FunctionEvalInfo *s, Sheet *sheet, int co
 FunctionEvalInfo *func_eval_info_cell (FunctionEvalInfo *s, Cell *cell);
 FunctionEvalInfo *func_eval_info_pos  (FunctionEvalInfo *s, const EvalPosition *fp);
 
-/*
- * Functions come in two fashions:  Those that only deal with
- * very specific data types and a constant number of arguments,
- * and those who don't.
- *
- * The former kind of functions receives a precomputed array of
- * Value pointers.
- *
- * The latter sort of functions receives the plain ExprNodes and
- * it is up to that routine to do the value computations and range
- * processing.
- */
-
-struct _FunctionDefinition {
-	char  const *name;
-	char  const *args;
-	char  const *named_arguments;
-	char  **help;
-	FuncType fn_type;
-	union {
-		FunctionNodes *fn_nodes;
-		FunctionArgs  *fn_args;
-	} fn;
-};
-
 /*
  * Built in / definable sheet names.
  */
@@ -170,14 +140,13 @@ struct _ExprName {
 	} t;
 };
 
-int         cell_ref_get_abs_col   (CellRef const *cell_ref,
-				    EvalPosition const *src_fp);
-int         cell_ref_get_abs_row   (CellRef const *cell_ref,
-				    EvalPosition const *src_fp);
-
-void        cell_get_abs_col_row   (const CellRef *cell_ref,
-				    int eval_col, int eval_row,
-				    int *col, int *row);
+int         cell_ref_get_abs_col   (CellRef const * const cell_ref,
+				    EvalPosition const * const src_fp);
+int         cell_ref_get_abs_row   (CellRef const * const cell_ref,
+				    EvalPosition const * const src_fp);
+void        cell_get_abs_col_row   (CellRef const * const cell_ref,
+				    CellPos const * const pos,
+				    int * const col, int * const row);
 
 ExprTree   *expr_parse_string      (const char *expr, const ParsePosition *pp,
 				    const char **desired_format, char **error_msg);
@@ -225,7 +194,11 @@ void expr_dump_tree (const ExprTree *tree);
  * Returns int(0) if the expression uses a non-existant cell for anything
  * other than an equality test.
  */
-Value       *eval_expr              (FunctionEvalInfo *s, ExprTree const *tree);
+Value       *eval_expr (FunctionEvalInfo * const s,
+			ExprTree const * const tree);
+
+Value       *expr_implicit_intersection (EvalPosition const * const pos,
+					 Value * const v);
 
 /* Setup of the symbol table */
 void         functions_init        (void);
