@@ -14,6 +14,19 @@
 #include "file.h"
 #include <locale.h>
 
+struct _FileOpener {
+	int             priority;
+	char           *format_description;
+	FileFormatProbe probe;
+	FileFormatOpen  open;
+};
+struct _FileSaver {
+	char            *extension;
+	char            *format_description;
+	FileFormatSave  save;
+};
+
+/* A GList of FileOpener structures */
 static GList *gnumeric_file_savers = NULL;
 static GList *gnumeric_file_openers = NULL;
 static FileSaver *current_saver = NULL;
@@ -136,29 +149,35 @@ file_format_unregister_save (FileFormatSave save)
 	}
 }
 
-gboolean
+/**
+ * workbook_load_from:
+ * @workbook: A blank workbook to load into.
+ * @filename: gives the URI of the file.
+ * 
+ * This function attempts to read the file into the supplied
+ * workbook.
+ * 
+ * Return value: success : NULL
+ *               failure : A string describing the error.  If the string is
+ *                         "" then a default error message should be used.  If the
+ *                         string is non empty it needs to be freed.
+ **/
+char *
 workbook_load_from (Workbook *wb, const char *filename)
 {
-	gboolean ret = FALSE;
 	GList *l;
 
-	for (l = gnumeric_file_openers; l; l = l->next){
-		const FileOpener *fo = l->data;
+	for (l = gnumeric_file_openers; l; l = l->next) {
+		FileOpener const * const fo = l->data;
 
-		if (fo->probe == NULL)
-			continue;
-		
-		if ((*fo->probe) (filename)){
-			
-			if ((*fo->open) (wb, filename)){
+		if (fo->probe != NULL && (*fo->probe) (filename)) {
+			char *result = (*fo->open) (wb, filename);
+			if (result == NULL)
 				workbook_mark_clean (wb);
-				ret = TRUE;
-			} else
-				ret = FALSE;
-			break;
+			return result;
 		}
 	}
-	return ret;
+	return "";
 }
 
 /**
@@ -171,19 +190,25 @@ workbook_load_from (Workbook *wb, const char *filename)
  *               NULL on failure.
  **/
 Workbook *
-workbook_try_read (const char *filename)
+workbook_try_read (const char *filename, char **msg)
 {
 	Workbook *wb;
+	char *result;
 
 	g_return_val_if_fail (filename != NULL, NULL);
 
 	wb = workbook_new ();
-	if (!workbook_load_from (wb, filename)) {
+	result = workbook_load_from (wb, filename);
+	if (result != NULL) {
 #ifdef ENABLE_BONOBO
 		gnome_object_destroy (GNOME_OBJECT (wb));
 #else
 		gtk_object_destroy   (GTK_OBJECT (wb));
 #endif
+		if (msg)
+			*msg = result;
+		else if (*result)
+			g_free (result);
 		wb = NULL;
 	}
 
@@ -191,11 +216,16 @@ workbook_try_read (const char *filename)
 }
 
 static void 
-file_error_message (char *message, const char *filename)
+file_error_message (char *message, const char *filename, char*plugin_mesg)
 {
-	char *s;
-	
-	s = g_strdup_printf (message, filename);
+	char *s = g_strdup_printf (message, filename);
+
+	if (plugin_mesg != NULL && *plugin_mesg) {
+		char *tmp = g_strconcat (s, "\n", plugin_mesg, NULL);
+		g_free (plugin_mesg);
+		g_free (s);
+		s = tmp;
+	}
 
 	gnumeric_notice (
 		NULL,
@@ -217,6 +247,7 @@ Workbook *
 workbook_read (const char *filename)
 {
 	Workbook *wb = NULL;
+	char *msg = NULL;
 
 	g_return_val_if_fail (filename != NULL, NULL);
 
@@ -227,8 +258,9 @@ workbook_read (const char *filename)
 		return wb;
 	}
 
-	wb = workbook_try_read (filename);
-	if (!wb) file_error_message (N_("Could not read file %s"), filename);
+	wb = workbook_try_read (filename, &msg);
+	if (!wb)
+		file_error_message (N_("Could not read file %s"), filename, msg);
 	return wb;
 }
 
@@ -309,8 +341,11 @@ workbook_import (Workbook *parent, const char *filename)
 			gtk_object_destroy   (GTK_OBJECT (wb));
 #endif
 			wb = NULL;
-		} else
+		} else {
 			workbook_mark_clean (wb);
+			/* We will want to change the name before saving */
+			wb->needs_name = TRUE;
+		}
 	}
 
 	if (ret != -1)
@@ -446,14 +481,15 @@ fs_key_event (GtkFileSelection *fsel, GdkEventKey *event)
 		return 0;
 }
 
-void
+gboolean
 workbook_save_as (Workbook *wb)
 {
 	GtkFileSelection *fsel;
 	gboolean accepted = FALSE;
 	GtkWidget *format_selector;
+	gboolean success  = FALSE;
 
-	g_return_if_fail (wb != NULL);
+	g_return_val_if_fail (wb != NULL, FALSE);
 
 	fsel = GTK_FILE_SELECTION (gtk_file_selection_new (_("Save workbook as")));
 
@@ -490,10 +526,10 @@ workbook_save_as (Workbook *wb)
 	gtk_grab_add (GTK_WIDGET (fsel));
 	gtk_main ();
 
-	if (accepted){
+	if (accepted) {
 		char *name = gtk_file_selection_get_filename (fsel);
 
-		if (name [strlen (name)-1] != '/'){
+		if (name [strlen (name)-1] != '/') {
 			char *base = g_basename (name);
 
 			current_saver = insure_saver (current_saver);
@@ -510,10 +546,11 @@ workbook_save_as (Workbook *wb)
 				if (current_saver->save (wb, name) == 0) {
 					workbook_set_filename (wb, name);
 					workbook_mark_clean (wb);
+					success = TRUE;
 				} else {
 					file_error_message 
 						(N_("Could not save to file %s"), 
-						 name);
+						 name, NULL);
 				}
 
 				g_free (name);
@@ -521,23 +558,26 @@ workbook_save_as (Workbook *wb)
 		}
 	}
 	gtk_widget_destroy (GTK_WIDGET (fsel));
+
+	return success;
 }
 
-void
+gboolean
 workbook_save (Workbook *wb)
 {
-	g_return_if_fail (wb != NULL);
+	g_return_val_if_fail (wb != NULL, FALSE);
 
-	if (wb->needs_name) {
-		workbook_save_as (wb);
-		return;
-	}
+	if (wb->needs_name)
+		return workbook_save_as (wb);
 
-	if (gnumeric_xml_write_workbook (wb, wb->filename) == 0)
+	if (gnumeric_xml_write_workbook (wb, wb->filename) == 0) {
 		workbook_mark_clean (wb);
-	else
+		return TRUE;
+	} else {
 		file_error_message (N_("Could not save to file %s"), 
-				    wb->filename);
+				    wb->filename, NULL);
+		return FALSE;
+	}
 }
 
 char *
