@@ -124,7 +124,7 @@ mstyle_hash (gconstpointer st)
 {
 	const MStyle *mstyle = (const MStyle *)st;
 	int     i;
-	guint32 hash;
+	guint32 hash = 0;
 
 	for (i = MSTYLE_ELEMENT_CONFLICT + 1; i < MSTYLE_ELEMENT_MAX; i++) {
 		const MStyleElement *e = &mstyle->elements [i];
@@ -154,13 +154,16 @@ mstyle_hash (gconstpointer st)
 		case MSTYLE_ANY_GUINT32:
 			hash = hash ^ e->u.any_guint32;
 			break;
+		case MSTYLE_ORIENTATION:
+			hash = hash ^ e->u.orientation;
+			break;
 		default:
 			g_warning ("Unimplemented hash item");
 			break;
 		}
 	}
 
-	return 0;
+	return hash;
 }
 
 static char *
@@ -483,6 +486,15 @@ mstyle_new_name (const gchar *name)
 
 static MStyle *default_mstyle = NULL;
 
+/**
+ * mstyle_new_default:
+ * 
+ * Return the default style,
+ * this should _never_ _ever_ have any of its elements
+ * set.
+ * 
+ * Return value: the default style.
+ **/
 MStyle *
 mstyle_new_default (void)
 {
