@@ -184,8 +184,11 @@ build_array (GList *cols)
 	return expr_tree_new_constant (array);
 }
 
+/* Make byacc happier */
+int yyparse(void);
 
-#line 167 "parser.y"
+
+#line 170 "parser.y"
 typedef union {
 	ExprTree *tree;
 	CellRef  *cell;
@@ -270,11 +273,11 @@ static const short yyrhs[] = {    38,
 
 #if YYDEBUG != 0
 static const short yyrline[] = { 0,
-   192,   194,   198,   201,   202,   203,   204,   205,   206,   207,
-   208,   209,   210,   211,   212,   213,   214,   215,   216,   217,
-   219,   220,   221,   223,   227,   232,   246,   252,   256,   261,
-   267,   271,   276,   279,   280,   283,   287,   295,   303,   306,
-   310,   315
+   195,   197,   201,   204,   205,   206,   207,   208,   209,   210,
+   211,   212,   213,   214,   215,   216,   217,   218,   219,   220,
+   222,   223,   224,   226,   230,   235,   249,   255,   259,   264,
+   270,   274,   279,   282,   283,   286,   290,   298,   306,   309,
+   313,   318
 };
 #endif
 
@@ -885,114 +888,114 @@ yyreduce:
   switch (yyn) {
 
 case 1:
-#line 192 "parser.y"
+#line 195 "parser.y"
 { *parser_result = yyvsp[0].tree; ;
     break;}
 case 2:
-#line 194 "parser.y"
+#line 197 "parser.y"
 {
 		*parser_result = build_array_formula (yyvsp[-12].tree, yyvsp[-7].tree, yyvsp[-9].tree, yyvsp[-1].tree, yyvsp[-4].tree) ;
 	;
     break;}
 case 3:
-#line 198 "parser.y"
+#line 201 "parser.y"
 { parser_error = PARSE_ERR_SYNTAX; ;
     break;}
 case 4:
-#line 201 "parser.y"
-{ yyval.tree = yyvsp[0].tree ;
+#line 204 "parser.y"
+{ yyval.tree = yyvsp[0].tree; ;
     break;}
 case 5:
-#line 202 "parser.y"
-{ yyval.tree = yyvsp[0].tree ;
+#line 205 "parser.y"
+{ yyval.tree = yyvsp[0].tree; ;
     break;}
 case 6:
-#line 203 "parser.y"
-{ yyval.tree = yyvsp[0].tree ;
+#line 206 "parser.y"
+{ yyval.tree = yyvsp[0].tree; ;
     break;}
 case 7:
-#line 204 "parser.y"
-{ yyval.tree = yyvsp[0].tree ;
+#line 207 "parser.y"
+{ yyval.tree = yyvsp[0].tree; ;
     break;}
 case 8:
-#line 205 "parser.y"
+#line 208 "parser.y"
 { yyval.tree = build_binop (yyvsp[-2].tree, OPER_ADD,       yyvsp[0].tree); ;
     break;}
 case 9:
-#line 206 "parser.y"
+#line 209 "parser.y"
 { yyval.tree = build_binop (yyvsp[-2].tree, OPER_SUB,       yyvsp[0].tree); ;
     break;}
 case 10:
-#line 207 "parser.y"
+#line 210 "parser.y"
 { yyval.tree = build_binop (yyvsp[-2].tree, OPER_MULT,      yyvsp[0].tree); ;
     break;}
 case 11:
-#line 208 "parser.y"
+#line 211 "parser.y"
 { yyval.tree = build_binop (yyvsp[-2].tree, OPER_DIV,       yyvsp[0].tree); ;
     break;}
 case 12:
-#line 209 "parser.y"
+#line 212 "parser.y"
 { yyval.tree = build_binop (yyvsp[-2].tree, OPER_EXP,       yyvsp[0].tree); ;
     break;}
 case 13:
-#line 210 "parser.y"
+#line 213 "parser.y"
 { yyval.tree = build_binop (yyvsp[-2].tree, OPER_CONCAT,    yyvsp[0].tree); ;
     break;}
 case 14:
-#line 211 "parser.y"
+#line 214 "parser.y"
 { yyval.tree = build_binop (yyvsp[-2].tree, OPER_EQUAL,     yyvsp[0].tree); ;
     break;}
 case 15:
-#line 212 "parser.y"
+#line 215 "parser.y"
 { yyval.tree = build_binop (yyvsp[-2].tree, OPER_LT,        yyvsp[0].tree); ;
     break;}
 case 16:
-#line 213 "parser.y"
+#line 216 "parser.y"
 { yyval.tree = build_binop (yyvsp[-2].tree, OPER_GT,        yyvsp[0].tree); ;
     break;}
 case 17:
-#line 214 "parser.y"
+#line 217 "parser.y"
 { yyval.tree = build_binop (yyvsp[-2].tree, OPER_GTE,       yyvsp[0].tree); ;
     break;}
 case 18:
-#line 215 "parser.y"
+#line 218 "parser.y"
 { yyval.tree = build_binop (yyvsp[-2].tree, OPER_NOT_EQUAL, yyvsp[0].tree); ;
     break;}
 case 19:
-#line 216 "parser.y"
+#line 219 "parser.y"
 { yyval.tree = build_binop (yyvsp[-2].tree, OPER_LTE,       yyvsp[0].tree); ;
     break;}
 case 20:
-#line 217 "parser.y"
+#line 220 "parser.y"
 { yyval.tree = yyvsp[-1].tree; ;
     break;}
 case 21:
-#line 219 "parser.y"
+#line 222 "parser.y"
 { yyval.tree = build_unary_op (OPER_PERCENT, yyvsp[-1].tree); ;
     break;}
 case 22:
-#line 220 "parser.y"
+#line 223 "parser.y"
 { yyval.tree = build_unary_op (OPER_NEG, yyvsp[0].tree); ;
     break;}
 case 23:
-#line 221 "parser.y"
+#line 224 "parser.y"
 { yyval.tree = yyvsp[0].tree; ;
     break;}
 case 24:
-#line 223 "parser.y"
+#line 226 "parser.y"
 {
 		yyval.tree = yyvsp[0].tree;
 	;
     break;}
 case 25:
-#line 227 "parser.y"
+#line 230 "parser.y"
 {
 		yyval.tree = build_array (yyvsp[-1].list);
 		forget_array (yyvsp[-1].list);
 	;
     break;}
 case 26:
-#line 232 "parser.y"
+#line 235 "parser.y"
 {
 		yyval.tree = p_new (ExprTree);
 		yyval.tree->ref_count  = 1;
@@ -1007,41 +1010,41 @@ case 26:
 	;
     break;}
 case 27:
-#line 246 "parser.y"
+#line 249 "parser.y"
 {
 		yyval.tree = yyvsp[-3].tree;
 		yyval.tree->u.function.arg_list = yyvsp[-1].list;
 	;
     break;}
 case 28:
-#line 252 "parser.y"
+#line 255 "parser.y"
 {
 		yyval.tree = yyvsp[0].tree;
 	;
     break;}
 case 29:
-#line 256 "parser.y"
+#line 259 "parser.y"
 {
 	        yyval.tree = yyvsp[0].tree;
 		yyval.tree->u.ref.sheet = yyvsp[-2].sheetref;
 	;
     break;}
 case 30:
-#line 261 "parser.y"
+#line 264 "parser.y"
 {
 	        yyval.tree = yyvsp[0].tree;
 		yyval.tree->u.ref.sheet = yyvsp[-2].sheetref;
 	;
     break;}
 case 31:
-#line 267 "parser.y"
+#line 270 "parser.y"
 {
 		yyval.list = g_list_prepend (NULL, yyvsp[0].tree);
 		alloc_glist (yyval.list);
         ;
     break;}
 case 32:
-#line 271 "parser.y"
+#line 274 "parser.y"
 {
 		forget_glist (yyvsp[0].list);
 		yyval.list = g_list_prepend (yyvsp[0].list, yyvsp[-2].tree);
@@ -1049,26 +1052,26 @@ case 32:
 	;
     break;}
 case 33:
-#line 276 "parser.y"
+#line 279 "parser.y"
 { yyval.list = NULL; ;
     break;}
 case 34:
-#line 279 "parser.y"
-{ yyval.tree = yyvsp[0].tree ;
+#line 282 "parser.y"
+{ yyval.tree = yyvsp[0].tree; ;
     break;}
 case 35:
-#line 280 "parser.y"
-{ yyval.tree = yyvsp[0].tree ;
+#line 283 "parser.y"
+{ yyval.tree = yyvsp[0].tree; ;
     break;}
 case 36:
-#line 283 "parser.y"
+#line 286 "parser.y"
 {
 		yyval.list = g_list_prepend (NULL, yyvsp[0].tree);
 		alloc_glist (yyval.list);
         ;
     break;}
 case 37:
-#line 287 "parser.y"
+#line 290 "parser.y"
 {
 		if (parser_array_col_separator == ',') {
 			forget_glist (yyvsp[0].list);
@@ -1079,7 +1082,7 @@ case 37:
 	;
     break;}
 case 38:
-#line 295 "parser.y"
+#line 298 "parser.y"
 {
 		if (parser_array_col_separator == '\\') {
 			forget_glist (yyvsp[0].list);
@@ -1090,18 +1093,18 @@ case 38:
 	;
     break;}
 case 39:
-#line 303 "parser.y"
+#line 306 "parser.y"
 { yyval.list = NULL; ;
     break;}
 case 40:
-#line 306 "parser.y"
+#line 309 "parser.y"
 {
 		yyval.list = g_list_prepend (NULL, yyvsp[0].list);
 		alloc_glist (yyval.list);
         ;
     break;}
 case 41:
-#line 310 "parser.y"
+#line 313 "parser.y"
 {
 		forget_glist (yyvsp[0].list);
 		yyval.list = g_list_prepend (yyvsp[0].list, yyvsp[-2].list);
@@ -1109,7 +1112,7 @@ case 41:
 	;
     break;}
 case 42:
-#line 315 "parser.y"
+#line 318 "parser.y"
 { yyval.list = NULL; ;
     break;}
 }
@@ -1310,7 +1313,7 @@ yyerrhandle:
   yystate = yyn;
   goto yynewstate;
 }
-#line 317 "parser.y"
+#line 320 "parser.y"
 
 
 static int
