@@ -21,7 +21,8 @@ static void sheet_selection_change (Sheet *sheet,
 
 /*
  * Quick utility routine to test intersect of line segments.
- * Returns : 4 --sA--sb--eb--eA--	a contains b
+ * Returns : 5 sA == sb eA == eb	a == b
+ *           4 --sA--sb--eb--eA--	a contains b
  *           3 --sA--sb--eA--eb--	overlap left
  *           2 --sb--sA--eA--eb--	b contains a
  *           1 --sb--sA--eb--eA--	overlap right
@@ -35,6 +36,11 @@ segments_intersect (int const s_a, int const e_a,
 	if (e_a < s_b || e_b < s_a)
 		return 0;
 
+	if (s_a == s_b)
+		return (e_a >= e_b) ? ((e_a == e_b) ? 5 : 4) : 2;
+	if (e_a == e_b)
+		return (s_a <= s_b) ? 4 : 2;
+
 	if (s_a < s_b)
 		return (e_a >= e_b) ? 4 : 3;
 
@@ -124,16 +130,16 @@ selection_first_range (Sheet const *sheet, gboolean const permit_complex)
 
 /*
  * selection_is_simple
- * @sheet : The sheet whose selection we are testing.
+ * @context      : The calling context to report errors to (GUI or corba)
+ * @sheet        : The sheet whose selection we are testing.
  * @command_name : A string naming the operation requiring a single range.
  *
  * This function tests to see if multiple ranges are selected.  If so it
- * produces a dialog box warning.
- *
- * TODO : Merge this with the proposed exception mechanism.
+ * produces a warning.
  */
 gboolean
-selection_is_simple (Sheet const *sheet, char const *command_name)
+selection_is_simple (CommandContext *context, Sheet const *sheet,
+		     char const *command_name)
 {
 	char *msg;
 
@@ -584,14 +590,16 @@ sheet_selection_unant (Sheet *sheet)
 }
 
 gboolean
-sheet_selection_copy (Sheet *sheet)
+sheet_selection_copy (CommandContext *context, Sheet *sheet)
 {
 	SheetSelection *ss;
 	g_return_val_if_fail (sheet != NULL, FALSE);
 	g_return_val_if_fail (IS_SHEET (sheet), FALSE);
 	g_return_val_if_fail (sheet->selections, FALSE);
 
-	if (!selection_is_simple (sheet, _("copy")))
+	/* FIXME FIXME : disable copying part of an array */
+
+	if (!selection_is_simple (context, sheet, _("copy")))
 		return FALSE;
 
 	ss = sheet->selections->data;
@@ -602,7 +610,7 @@ sheet_selection_copy (Sheet *sheet)
 }
 
 gboolean
-sheet_selection_cut (Sheet *sheet)
+sheet_selection_cut (CommandContext *context, Sheet *sheet)
 {
 	SheetSelection *ss;
 
@@ -622,10 +630,16 @@ sheet_selection_cut (Sheet *sheet)
 	g_return_val_if_fail (IS_SHEET (sheet), FALSE);
 	g_return_val_if_fail (sheet->selections, FALSE);
 
-	if (!selection_is_simple (sheet, _("cut")))
+	if (!selection_is_simple (context, sheet, _("cut")))
 		return FALSE;
 
 	ss = sheet->selections->data;
+	if (!sheet_check_for_partial_array (sheet,
+					    ss->user.start.row,ss->user.start.col,
+					    ss->user.end.row, ss->user.end.col)) {
+		gnumeric_no_modify_array_notice (sheet->workbook);
+		return FALSE;
+	}
 
 	application_clipboard_cut (sheet, &ss->user);
 
@@ -653,7 +667,8 @@ sheet_selection_move (struct expr_relocate_info *rinfo)
 }
 
 void
-sheet_selection_paste (Sheet *sheet, int dest_col, int dest_row,
+sheet_selection_paste (CommandContext *context, Sheet *sheet,
+		       int dest_col, int dest_row,
 		       int paste_flags, guint32 time)
 {
 	CellRegion  *content;
@@ -662,7 +677,7 @@ sheet_selection_paste (Sheet *sheet, int dest_col, int dest_row,
 	g_return_if_fail (sheet != NULL);
 	g_return_if_fail (IS_SHEET (sheet));
 
-	if (!selection_is_simple (sheet, _("paste")))
+	if (!selection_is_simple (context, sheet, _("paste")))
 		return;
 
 	area = application_clipboard_area_get ();
@@ -677,9 +692,9 @@ sheet_selection_paste (Sheet *sheet, int dest_col, int dest_row,
 		g_return_if_fail (sel != NULL);
 		g_return_if_fail (area != NULL);
 
-		/* TODO FIXME : This should not be a dialog.
-		 * It should be an exception or a dialog depending
-		 * on the caller.
+		/* FIXME FIXME : This should not be a dialog.
+		 *
+		 * Use the context
 		 */
 		if (!range_is_singleton (sel) &&
 		    ((sel->end.col - sel->start.col) != (area->end.col - area->start.col) ||
@@ -703,15 +718,24 @@ sheet_selection_paste (Sheet *sheet, int dest_col, int dest_row,
 		rinfo.origin_sheet = src_sheet;
 		rinfo.target_sheet = sheet;
 
-		sheet_move_range      (&rinfo);
+		sheet_move_range      (NULL, &rinfo);
 		sheet_selection_move  (&rinfo);
 		application_clipboard_clear ();
 	} else
-		clipboard_paste_region (content, sheet, dest_col, dest_row,
+		clipboard_paste_region (context, content,
+					sheet, dest_col, dest_row,
 					paste_flags, time);
 }
 
-/* TODO TODO TODO : Remove these and just call the functions directly */
+static void
+cb_sheet_selection_clear (Sheet *sheet, 
+			  int start_col, int start_row,
+			  int end_col,   int end_row,
+			  void *context)
+{
+	sheet_clear_region (context, sheet, start_col, start_row,
+			    end_col,  end_row, TRUE);
+}
 
 /**
  * sheet_selection_clear:
@@ -720,9 +744,20 @@ sheet_selection_paste (Sheet *sheet, int dest_col, int dest_row,
  * Removes the contents and styles.
  **/
 void
-sheet_selection_clear (Sheet *sheet)
+sheet_selection_clear (CommandContext *context, Sheet *sheet)
 {
-	selection_apply (sheet, &sheet_clear_region, TRUE, NULL);
+	selection_apply (sheet, &cb_sheet_selection_clear, TRUE, NULL);
+}
+
+static void
+cb_sheet_selection_clear_content (Sheet *sheet, 
+				  int start_col, int start_row,
+				  int end_col,   int end_row,
+				  void *context)
+{
+	sheet_clear_region_content (context, sheet, 
+				    start_col, start_row,
+				    end_col,   end_row);
 }
 
 /**
@@ -732,9 +767,10 @@ sheet_selection_clear (Sheet *sheet)
  * Removes the contents of all the cells in the current selection.
  **/
 void
-sheet_selection_clear_content (Sheet *sheet)
+sheet_selection_clear_content (CommandContext *context, Sheet *sheet)
 {
-	selection_apply (sheet, &sheet_clear_region_content, TRUE, NULL);
+	selection_apply (sheet, &cb_sheet_selection_clear_content,
+			 TRUE, context);
 	sheet_load_cell_val(sheet);
 }
 
@@ -745,7 +781,7 @@ sheet_selection_clear_content (Sheet *sheet)
  * Removes all of the comments on the range of selected cells.
  **/
 void
-sheet_selection_clear_comments (Sheet *sheet)
+sheet_selection_clear_comments (CommandContext *context, Sheet *sheet)
 {
 	selection_apply (sheet, &sheet_clear_region_comments, TRUE, NULL);
 }
@@ -757,7 +793,7 @@ sheet_selection_clear_comments (Sheet *sheet)
  * Removes all formating
  **/
 void
-sheet_selection_clear_formats (Sheet *sheet)
+sheet_selection_clear_formats (CommandContext *context, Sheet *sheet)
 {
 	selection_apply (sheet, &sheet_clear_region_formats, TRUE, NULL);
 }
@@ -796,6 +832,11 @@ selection_apply (Sheet *sheet, SelectionApplyFunc const func,
 		return;
 	}
 
+#undef DEBUG_SELECTION
+#ifdef DEBUG_SELECTION
+	fprintf (stderr, "============================\n");
+#endif
+
 	/*
 	 * Run through all the selection regions to see if any of
 	 * the proposed regions overlap.  Start the search with the
@@ -826,12 +867,20 @@ selection_apply (Sheet *sheet, SelectionApplyFunc const func,
 				continue;
 			}
 
+
+#ifdef DEBUG_SELECTION
+			fprintf (stderr, "a = ");
+			range_dump (a);
+			fprintf (stderr, "b = ");
+			range_dump (b);
+#endif
+
 			col_intersect =
 				segments_intersect (a->start.col, a->end.col,
 						    b->start.col, b->end.col);
 
-#if 0
-			printf ("col = %d\na = %d -> %d\nb = %d -> %d\n",
+#ifdef DEBUG_SELECTION
+			fprintf (stderr, "col = %d\na = %d -> %d\nb = %d -> %d\n",
 				col_intersect,
 				a->start.col, a->end.col,
 				b->start.col, b->end.col);
@@ -846,8 +895,8 @@ selection_apply (Sheet *sheet, SelectionApplyFunc const func,
 			row_intersect =
 				segments_intersect (a->start.row, a->end.row,
 						    b->start.row, b->end.row);
-#if 0
-			printf ("row = %d\na = %d -> %d\nb = %d -> %d\n",
+#ifdef DEBUG_SELECTION
+			fprintf (stderr, "row = %d\na = %d -> %d\nb = %d -> %d\n",
 				row_intersect,
 				a->start.row, a->end.row,
 				b->start.row, b->end.row);
@@ -859,6 +908,17 @@ selection_apply (Sheet *sheet, SelectionApplyFunc const func,
 				continue;
 			}
 
+			/* Simplify our lives by allowing equality to work in our favour */
+			if (col_intersect == 5) {
+				if (row_intersect == 5)
+					row_intersect = 4;
+				if (row_intersect == 4 || row_intersect == 2)
+					col_intersect = row_intersect;
+			} else if (row_intersect == 5) {
+				if (col_intersect == 4 || col_intersect == 2)
+					row_intersect = col_intersect;
+			}
+
 			/* Cross product of intersection cases */
 			switch (col_intersect) {
 			case 4 : /* a contains b */
@@ -1035,6 +1095,10 @@ selection_apply (Sheet *sheet, SelectionApplyFunc const func,
 		Range *r = proposed->data;
 		proposed = g_slist_remove (proposed, r);
 
+#ifdef DEBUG_SELECTION
+		range_dump (r);
+#endif
+
 		(*func) (sheet,
 			 r->start.col, r->start.row,
 			 r->end.col, r->end.row,
