@@ -58,11 +58,11 @@ get_file_name (void)
 	
         gtk_signal_connect (GTK_OBJECT (fs->ok_button),
                             "clicked",
-                            GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
+                            GTK_SIGNAL_FUNC (gtk_main_quit), NULL);
 
         gtk_signal_connect (GTK_OBJECT (fs->cancel_button),
                             "clicked",
-                            GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
+                            GTK_SIGNAL_FUNC (gtk_main_quit), NULL);
 
 	gtk_widget_show (GTK_WIDGET (fs));
 	gtk_main ();
@@ -164,11 +164,8 @@ sheet_object_bonobo_load (SheetObjectBonobo *sob,
 	
 	g_return_val_if_fail (sob != NULL, FALSE);
 	g_return_val_if_fail (IS_SHEET_OBJECT_BONOBO (sob), FALSE);
-	g_return_val_if_fail (sob->client_site == NULL, FALSE);
+	g_return_val_if_fail (sob->client_site != NULL, FALSE);
 	
-	sob->client_site = gnome_client_site_new (
-		SHEET_OBJECT (sob)->sheet->workbook->gnome_container);
-
 	CORBA_exception_init (&ev);
 	
 	ret = GNOME_Unknown_query_interface (
