@@ -22,6 +22,7 @@
 #include "print-info.h"
 #include "mstyle.h"
 #include "application.h"
+#include "command-context.h"
 #ifdef ENABLE_BONOBO
 #    include <libgnorba/gnorba.h>
 #endif
@@ -218,7 +219,8 @@ sheet_new (Workbook *wb, const char *name)
 
 	sheet_corba_setup (sheet);
 
-	sheet_set_dirty (sheet, FALSE);
+	sheet->pristine = TRUE;
+	sheet->modified = FALSE;
 
 	return sheet;
 }
@@ -505,6 +507,119 @@ sheet_reposition_comments_from_col (Sheet *sheet, int col)
 	}
 }
 
+/**
+ * sheet_cell_get:
+ * @sheet:  The sheet where we want to locate the cell
+ * @col:    the cell column
+ * @row:    the cell row
+ *
+ * Return value: a (Cell *) containing the Cell, or NULL if
+ * the cell does not exist
+ */
+inline Cell *
+sheet_cell_get (Sheet const *sheet, int col, int row)
+{
+	Cell *cell;
+	CellPos cellpos;
+
+	g_return_val_if_fail (sheet != NULL, NULL);
+	g_return_val_if_fail (IS_SHEET (sheet), NULL);
+
+	cellpos.col = col;
+	cellpos.row = row;
+	cell = g_hash_table_lookup (sheet->cell_hash, &cellpos);
+
+	return cell;
+}
+
+/**
+ * sheet_cell_fetch:
+ * @sheet:  The sheet where we want to locate the cell
+ * @col:    the cell column
+ * @row:    the cell row
+ *
+ * Return value: a (Cell *) containing the Cell at col, row.
+ * If no cell existed at that location before, it is created.
+ */
+inline Cell *
+sheet_cell_fetch (Sheet *sheet, int col, int row)
+{
+	Cell *cell;
+
+	g_return_val_if_fail (sheet != NULL, NULL);
+	g_return_val_if_fail (IS_SHEET (sheet), NULL);
+
+	cell = sheet_cell_get (sheet, col, row);
+	if (!cell)
+		cell = sheet_cell_new (sheet, col, row);
+
+	return cell;
+}
+
+/**
+ * sheet_get_extent_cb:
+ * 
+ * checks the cell to see if should be used to calculate sheet extent
+ **/
+static void
+sheet_get_extent_cb (gpointer key, gpointer value, gpointer data)
+{
+	Cell *cell = (Cell *) value;
+	
+	if ( cell && !cell_is_blank (cell)) {
+		Range *range = (Range *)data;
+
+		if ( cell->row->pos < range->start.row)
+			range->start.row = cell->row->pos;
+
+		if ( cell->row->pos > range->end.row)
+			range->end.row = cell->row->pos;
+
+		if ( cell->col->pos < range->start.col)
+			range->start.col = cell->col->pos;
+
+		if ( cell->col->pos > range->end.col)
+			range->end.col = cell->col->pos;
+	}
+}
+
+/**
+ * sheet_get_extent:
+ * @sheet: the sheet
+ * 
+ * calculates the area occupied by cell data.
+ * 
+ * Return value: the range.
+ **/
+Range
+sheet_get_extent (Sheet const *sheet)
+{
+	Range r;
+
+	r.start.col = SHEET_MAX_COLS - 2;
+	r.start.row = SHEET_MAX_ROWS - 2;
+	r.end.col   = 0;
+	r.end.row   = 0;
+
+	g_return_val_if_fail (sheet != NULL, r);
+	g_return_val_if_fail (IS_SHEET (sheet), r);
+
+	g_hash_table_foreach(sheet->cell_hash, sheet_get_extent_cb, &r);
+
+	if (r.start.col >= SHEET_MAX_COLS - 2)
+		r.start.col = 0;
+	if (r.start.row >= SHEET_MAX_ROWS - 2)
+		r.start.row = 0;
+	if (r.end.col < 0)
+		r.end.col = 0;
+	if (r.end.row < 0)
+		r.end.row = 0;
+
+	sheet_style_get_extent (&r, sheet);
+
+	return r;
+}
+
 void
 sheet_row_info_set_height (Sheet *sheet, ColRowInfo *ri, int height, gboolean height_set_by_user)
 {
@@ -895,6 +1010,76 @@ sheet_col_get_distance (Sheet const *sheet, int from, int to)
 	return pixels;
 }
 
+/**
+ * sheet_get_default_external_col_width:
+ *
+ * Return the default number of units in a column, including margins.
+ * This function returns the raw sum, no rounding etc.
+ */
+double
+sheet_get_default_external_col_width (Sheet const *sheet)
+{
+	ColRowInfo const *ci;
+
+	g_assert (sheet != NULL);
+ 
+	ci = &sheet->cols.default_style;
+	return  ci->units + ci->margin_a_pt + ci->margin_b_pt;
+}
+
+/**
+ * sheet_get_default_external_row_height:
+ *
+ * Return the default number of units in a row, including margins.
+ * This function returns the raw sum, no rounding etc.
+ */
+double
+sheet_get_default_external_row_height (Sheet const *sheet)
+{
+	ColRowInfo const *ci;
+
+	g_assert (sheet != NULL);
+ 
+	ci = &sheet->rows.default_style;
+	return  ci->units + ci->margin_a_pt + ci->margin_b_pt;
+}
+
+/**
+ * sheet_col_get_external_height:
+ *
+ * Return the number of units in a row, including margins.
+ * This function returns the raw sum. If you need rounding and
+ * fiddling, use sheet_col_get_unit_distance.
+ */
+double
+sheet_col_get_external_width (Sheet const *sheet, int const pos)
+{
+	ColRowInfo const *ci;
+
+	g_assert (sheet != NULL);
+ 
+	ci = sheet_col_get_info (sheet, pos);
+	return  ci->units + ci->margin_a_pt + ci->margin_b_pt;
+}
+
+/**
+ * sheet_row_get_external_height:
+ *
+ * Return the number of units in a row, including margins.
+ * This function returns the raw sum. If you need rounding and
+ * fiddling, use sheet_row_get_unit_distance.
+ */
+double
+sheet_row_get_external_height (Sheet const *sheet, int const pos)
+{
+	ColRowInfo const *ci;
+
+	g_assert (sheet != NULL);
+ 
+	ci = sheet_row_get_info (sheet, pos);
+	return  ci->units + ci->margin_a_pt + ci->margin_b_pt;
+}
+
 /**
  * sheet_row_get_distance:
  *
@@ -1544,8 +1729,8 @@ sheet_is_cell_array (Sheet *sheet, int const col, int const row)
  *
  * returns FALSE is an array would be divided.
  */
-static gboolean
-range_check_for_partial_array (Sheet *sheet,
+gboolean
+sheet_check_for_partial_array (Sheet *sheet,
 			       int const start_row, int const start_col,
 			       int end_row, int end_col)
 {
@@ -1817,55 +2002,6 @@ sheet_row_fetch (Sheet *sheet, int pos)
 	return res;
 }
 
-/**
- * sheet_cell_get:
- * @sheet:  The sheet where we want to locate the cell
- * @col:    the cell column
- * @row:    the cell row
- *
- * Return value: a (Cell *) containing the Cell, or NULL if
- * the cell does not exist
- */
-Cell *
-sheet_cell_get (Sheet const *sheet, int col, int row)
-{
-	Cell *cell;
-	CellPos cellpos;
-
-	g_return_val_if_fail (sheet != NULL, NULL);
-	g_return_val_if_fail (IS_SHEET (sheet), NULL);
-
-	cellpos.col = col;
-	cellpos.row = row;
-	cell = g_hash_table_lookup (sheet->cell_hash, &cellpos);
-
-	return cell;
-}
-
-/**
- * sheet_cell_fetch:
- * @sheet:  The sheet where we want to locate the cell
- * @col:    the cell column
- * @row:    the cell row
- *
- * Return value: a (Cell *) containing the Cell at col, row.
- * If no cell existed at that location before, it is created.
- */
-Cell *
-sheet_cell_fetch (Sheet *sheet, int col, int row)
-{
-	Cell *cell;
-
-	g_return_val_if_fail (sheet != NULL, NULL);
-	g_return_val_if_fail (IS_SHEET (sheet), NULL);
-
-	cell = sheet_cell_get (sheet, col, row);
-	if (!cell)
-		cell = sheet_cell_new (sheet, col, row);
-
-	return cell;
-}
-
 #define SWAP_INT(a,b) do { int t; t = a; a = b; b = t; } while (0)
 
 /**
@@ -2393,14 +2529,15 @@ assemble_clear_cell_list (Sheet *sheet, int col, int row, Cell *cell,
  *
  * Clears are region of cells
  *
- * @keepStyles : If this is non-null then styles are not erased.
+ * @clearStyles : If this is TRUE then styles are erased.
  *
  * We assemble a list of cells to destroy, since we will be making changes
  * to the structure being manipulated by the sheet_cell_foreach_range routine
  */
 void
-sheet_clear_region (Sheet *sheet, int start_col, int start_row, int end_col, int end_row,
-		    void *keepStyles)
+sheet_clear_region (CommandContext *context, Sheet *sheet,
+		    int start_col, int start_row, int end_col, int end_row,
+		    gboolean const clearStyles)
 {
 	struct sheet_clear_region_callback_data cb;
 	GList *l;
@@ -2420,7 +2557,7 @@ sheet_clear_region (Sheet *sheet, int start_col, int start_row, int end_col, int
 	cb.l = NULL;
 
 	/* Clear the style in the region (new_default will ref the style for us). */
-	if (!keepStyles)
+	if (clearStyles)
 		sheet_style_attach (sheet, cb.r, mstyle_new_default ());
 
 	if (sheet_cell_foreach_range (sheet, TRUE,
@@ -2438,18 +2575,10 @@ sheet_clear_region (Sheet *sheet, int start_col, int start_row, int end_col, int
 		cell_thaw_redraws ();
 		workbook_recalc (sheet->workbook);
 	} else 
-		gnumeric_no_modify_array_notice (sheet->workbook);
+		gnumeric_error_splits_array (context);
 	g_list_free (cb.l);
 }
 
-static Value *
-clear_cell_content (Sheet *sheet, int col, int row, Cell *cell, void *user_data)
-{
-	cell_set_text (cell, "");
-
-	return NULL;
-}
-
 /**
  * sheet_clear_region_content:
  * @sheet:     The sheet on which we operate
@@ -2461,24 +2590,13 @@ clear_cell_content (Sheet *sheet, int col, int row, Cell *cell, void *user_data)
  * Clears the contents in a region of cells
  */
 void
-sheet_clear_region_content (Sheet *sheet, int start_col, int start_row, int end_col, int end_row,
-			    void *closure)
+sheet_clear_region_content (CommandContext *context, Sheet *sheet,
+			    int start_col, int start_row,
+			    int end_col, int end_row)
 {
-	g_return_if_fail (sheet != NULL);
-	g_return_if_fail (IS_SHEET (sheet));
-	g_return_if_fail (start_col <= end_col);
-	g_return_if_fail (start_row <= end_row);
-
-	/* Queue a redraw for the region being redrawn */
-	sheet_redraw_cell_region (sheet, start_col, start_row, end_col, end_row);
-
-	sheet_cell_foreach_range (
-		sheet, TRUE,
-		start_col, start_row,
-		end_col, end_row,
-		clear_cell_content, NULL);
-
-	workbook_recalc (sheet->workbook);
+	sheet_clear_region (context, sheet,
+			    start_col, start_row,
+			    end_col, end_row, FALSE);
 }
 
 static Value *
@@ -2654,10 +2772,14 @@ sheet_fill_selection_with (Sheet *sheet, const char *str,
 	for (l = sheet->selections; l; l = l->next)
 	{
 		SheetSelection *ss = l->data;
-		if (!range_check_for_partial_array (sheet,
+		if (!sheet_check_for_partial_array (sheet,
 						    ss->user.start.row,ss->user.start.col,
 						    ss->user.end.row, ss->user.end.col)) {
+#if 0
+			gnumeric_error_splits_array (context);
+#else
 			gnumeric_no_modify_array_notice (sheet->workbook);
+#endif
 			return;
 		}
 	}
@@ -2936,6 +3058,9 @@ sheet_mark_clean (Sheet *sheet)
 	g_return_if_fail (sheet != NULL);
 	g_return_if_fail (IS_SHEET (sheet));
 
+	if (sheet->modified)
+		sheet->pristine = FALSE;
+
 	sheet->modified = FALSE;
 }
 
@@ -2945,9 +3070,29 @@ sheet_set_dirty (Sheet *sheet, gboolean is_dirty)
 	g_return_if_fail (sheet != NULL);
 	g_return_if_fail (IS_SHEET (sheet));
 
+	if (sheet->modified)
+		sheet->pristine = FALSE;
+
 	sheet->modified = is_dirty;
 }
 
+/**
+ * sheet_is_pristine:
+ * @sheet: 
+ * 
+ * Sees if the sheet has ever been touched.
+ * 
+ * Return value: TRUE if it is perfectly clean.
+ **/
+gboolean
+sheet_is_pristine (Sheet *sheet)
+{
+	g_return_val_if_fail (sheet != NULL, FALSE);
+	g_return_val_if_fail (IS_SHEET (sheet), FALSE);
+
+	return sheet->pristine && !sheet->modified;
+}
+
 /**
  * sheet_lookup_by_name:
  * @name:  a sheet name.
@@ -3031,11 +3176,6 @@ sheet_set_selection (Sheet *sheet, SheetSelection const *ss)
 
 /****************************************************************************/
 
-/*
- * FIXME FIXME FIXME : 1999/Sept/29 when the style changes are done we need
- *  to figure out how row/col/cell movement will apply to that.
- */
-
 /*
  * Callback for sheet_cell_foreach_range to remove a cell and
  * put it in a temporary list.
@@ -3145,7 +3285,8 @@ colrow_move (Sheet *sheet,
  * @count   The number of columns to be inserted
  */
 void
-sheet_insert_col (Sheet *sheet, int col, int count)
+sheet_insert_cols (CommandContext *context, Sheet *sheet,
+		   int col, int count)
 {
 	struct expr_relocate_info reloc_info;
 	GList *deps;
@@ -3165,7 +3306,7 @@ sheet_insert_col (Sheet *sheet, int col, int count)
 					      col, SHEET_MAX_ROWS-1,
 					      &avoid_dividing_array_horizontal,
 					      NULL) != NULL){
-			gnumeric_no_modify_array_notice (sheet->workbook);
+			gnumeric_error_splits_array (context);
 			return;
 		}
 
@@ -3207,13 +3348,14 @@ sheet_insert_col (Sheet *sheet, int col, int count)
 }
 
 /*
- * sheet_delete_col
+ * sheet_delete_cols
  * @sheet   The sheet
  * @col     At which position we want to start deleting columns
  * @count   The number of columns to be deleted
  */
 void
-sheet_delete_col (Sheet *sheet, int col, int count)
+sheet_delete_cols (CommandContext *context, Sheet *sheet,
+		   int col, int count)
 {
 	struct expr_relocate_info reloc_info;
 	GList *deps;
@@ -3228,10 +3370,10 @@ sheet_delete_col (Sheet *sheet, int col, int count)
 		return;
 
 	/* 0. Walk cells in deleted cols and ensure arrays aren't divided. */
-	if (!range_check_for_partial_array (sheet, 0, col, 
+	if (!sheet_check_for_partial_array (sheet, 0, col, 
 					    SHEET_MAX_ROWS-1, col+count-1))
 	{
-		gnumeric_no_modify_array_notice (sheet->workbook);
+		gnumeric_error_splits_array (context);
 		return;
 	}
 
@@ -3276,13 +3418,14 @@ sheet_delete_col (Sheet *sheet, int col, int count)
 }
 
 /**
- * sheet_insert_row:
+ * sheet_insert_rows:
  * @sheet   The sheet
  * @row     At which position we want to insert
  * @count   The number of rows to be inserted
  */
 void
-sheet_insert_row (Sheet *sheet, int row, int count)
+sheet_insert_rows (CommandContext *context, Sheet *sheet,
+		   int row, int count)
 {
 	struct expr_relocate_info reloc_info;
 	GList *deps;
@@ -3302,8 +3445,8 @@ sheet_insert_row (Sheet *sheet, int row, int count)
 					      0, row,
 					      SHEET_MAX_COLS-1, row,
 					      &avoid_dividing_array_vertical,
-					      NULL) != NULL){
-			gnumeric_no_modify_array_notice (sheet->workbook);
+					      NULL) != NULL) {
+			gnumeric_error_splits_array (context);
 			return;
 		}
 
@@ -3345,13 +3488,14 @@ sheet_insert_row (Sheet *sheet, int row, int count)
 }
 
 /*
- * sheet_delete_row
+ * sheet_delete_rows
  * @sheet   The sheet
  * @row     At which position we want to start deleting rows
  * @count   The number of rows to be deleted
  */
 void
-sheet_delete_row (Sheet *sheet, int row, int count)
+sheet_delete_rows (CommandContext *context, Sheet *sheet,
+		   int row, int count)
 {
 	struct expr_relocate_info reloc_info;
 	GList *deps;
@@ -3366,10 +3510,10 @@ sheet_delete_row (Sheet *sheet, int row, int count)
 		return;
 
 	/* 0. Walk cells in deleted rows and ensure arrays aren't divided. */
-	if (!range_check_for_partial_array (sheet, row, 0, 
+	if (!sheet_check_for_partial_array (sheet, row, 0, 
 					    row+count-1, SHEET_MAX_COLS-1))
 	{
-		gnumeric_no_modify_array_notice (sheet->workbook);
+		gnumeric_error_splits_array (context);
 		return;
 	}
 
@@ -3418,12 +3562,12 @@ sheet_delete_row (Sheet *sheet, int row, int count)
  * do that ?
  */
 void
-sheet_move_range (struct expr_relocate_info const * rinfo)
+sheet_move_range (CommandContext *context,
+		  struct expr_relocate_info const * rinfo)
 {
 	GList *deps, *cells = NULL;
 	Cell  *cell;
 	gboolean inter_sheet_formula;
-	int	dummy;
 
 	g_return_if_fail (rinfo->origin_sheet != NULL);
 	g_return_if_fail (IS_SHEET (rinfo->origin_sheet));
@@ -3450,14 +3594,13 @@ sheet_move_range (struct expr_relocate_info const * rinfo)
 	cells = g_list_reverse (cells);
 
 	/* 4. Clear the target area */
-	/* Pass a pointer, any pointer, rather than NULL. That flags
-	 * not to clear styles */
-	sheet_clear_region (rinfo->target_sheet, 
+	sheet_clear_region (context,
+			    rinfo->target_sheet, 
 			    rinfo->origin.start.col + rinfo->col_offset,
 			    rinfo->origin.start.row + rinfo->row_offset,
 			    rinfo->origin.end.col + rinfo->col_offset,
 			    rinfo->origin.end.row + rinfo->row_offset,
-			    &dummy);
+			    FALSE); /* Do not to clear styles */
 
 	/* Insert the cells back */
 	for (; cells != NULL ; cells = g_list_remove (cells, cell)) {
@@ -3521,7 +3664,8 @@ sheet_move_range (struct expr_relocate_info const * rinfo)
  */
 
 void
-sheet_shift_rows (Sheet *sheet, int col, int start_row, int end_row, int count)
+sheet_shift_rows (CommandContext *context, Sheet *sheet,
+		  int col, int start_row, int end_row, int count)
 {
 	struct expr_relocate_info rinfo;
 	rinfo.origin.start.col = col;
@@ -3532,7 +3676,7 @@ sheet_shift_rows (Sheet *sheet, int col, int start_row, int end_row, int count)
 	rinfo.col_offset = count;
 	rinfo.row_offset = 0;
 
-	sheet_move_range (&rinfo);
+	sheet_move_range (context, &rinfo);
 }
 
 /**
@@ -3549,7 +3693,8 @@ sheet_shift_rows (Sheet *sheet, int col, int start_row, int end_row, int count)
  * and copies them @count units (possibly negative) downwards.
  */
 void
-sheet_shift_cols (Sheet *sheet, int start_col, int end_col, int row, int count)
+sheet_shift_cols (CommandContext *context, Sheet *sheet,
+		  int start_col, int end_col, int row, int count)
 {
 	struct expr_relocate_info rinfo;
 	rinfo.origin.start.col = start_col;
@@ -3560,5 +3705,5 @@ sheet_shift_cols (Sheet *sheet, int start_col, int end_col, int row, int count)
 	rinfo.col_offset = 0;
 	rinfo.row_offset = count;
 
-	sheet_move_range (&rinfo);
+	sheet_move_range (context, &rinfo);
 }
