@@ -18,6 +18,7 @@
 #include "widgets/widget-color-combo.h"
 #include "widgets/gnumeric-toolbar.h"
 #include "workbook-private.h"
+#include "application.h"
 #include "format.h"
 
 /*
@@ -41,7 +42,7 @@ set_selection_halign (Workbook *wb, StyleHAlignFlags align)
 	Sheet  *sheet;
 
 	sheet = workbook_get_current_sheet (wb);
-	sheet_selection_unant (sheet);
+	application_clipboard_clear ();
 
 	mstyle = mstyle_new ();
 	mstyle_set_align_h (mstyle, align);
@@ -81,7 +82,7 @@ change_selection_font (Workbook *wb, int bold, int italic)
 	Sheet  *sheet;
 
 	sheet = workbook_get_current_sheet (wb);
-	sheet_selection_unant (sheet);
+	application_clipboard_clear ();
 
 	if (bold >= 0) {
 		mstyle = mstyle_new ();
@@ -312,13 +313,17 @@ back_color_changed (ColorCombo *cc, GdkColor *color, int color_index, Workbook *
 	Sheet  *sheet = workbook_get_current_sheet (wb);
 	MStyle *mstyle = mstyle_new ();
 
-	/* We need to have a pattern of at least solid to draw a background colour */
-	if (!mstyle_is_element_set  (mstyle, MSTYLE_PATTERN) ||
-	    mstyle_get_pattern (mstyle) < 1)
-		mstyle_set_pattern (mstyle, 1);
+	if (color_index >= 0) {
+		/* We need to have a pattern of at least solid to draw a background colour */
+		if (!mstyle_is_element_set  (mstyle, MSTYLE_PATTERN) ||
+		    mstyle_get_pattern (mstyle) < 1)
+			mstyle_set_pattern (mstyle, 1);
 
-	mstyle_set_color (mstyle, MSTYLE_COLOR_BACK, 
-			  style_color_new (color->red, color->green, color->blue));
+		mstyle_set_color (mstyle, MSTYLE_COLOR_BACK, 
+				  style_color_new (color->red, color->green, color->blue));
+	} else
+		/* Set background to NONE */
+		mstyle_set_pattern (mstyle, 0);
 
 	sheet_selection_apply_style (sheet, mstyle);
 }
