@@ -30,6 +30,8 @@
 #include "widgets/gnumeric-toolbar.h"
 #include "workbook-cmd-format.h"
 #include "workbook-format-toolbar.h"
+#include "workbook-view.h"
+#include "command-context-gui.h"
 
 #ifdef ENABLE_BONOBO
 #include <bonobo/gnome-persist-file.h>
@@ -71,8 +73,22 @@ file_open_cmd (GtkWidget *widget, Workbook *wb)
 	char *fname = dialog_query_load_file (wb);
 	Workbook *new_wb;
 
-	if (fname && (new_wb = workbook_read (fname)))
+	if (!fname)
+		return;
+
+	new_wb = workbook_read (fname);
+
+	if (new_wb != NULL) {
 		gtk_widget_show (new_wb->toplevel);
+
+		if (workbook_is_pristine (wb)) {
+#ifdef ENABLE_BONOBO
+			gnome_object_unref (GNOME_OBJECT (wb));
+#else
+			gtk_object_unref (GTK_OBJECT (wb));
+#endif
+		}
+	}
 	g_free (fname);
 }
 
@@ -81,13 +97,23 @@ file_import_cmd (GtkWidget *widget, Workbook *wb)
 {
 	char *fname = dialog_query_load_file (wb);
 	Workbook *new_wb;
-
+	
 	if (!fname)
 		return;
+	
+	new_wb = workbook_import (wb, fname);
+	if (new_wb) {
+		gtk_widget_show (new_wb->toplevel);
 
-       new_wb = workbook_import (wb, fname);
-       if (new_wb)
-	       gtk_widget_show (new_wb->toplevel);
+		if (workbook_is_pristine (wb)) {
+#ifdef ENABLE_BONOBO
+			gnome_object_unref (GNOME_OBJECT (wb));
+#else
+			gtk_object_unref (GTK_OBJECT (wb));
+#endif
+		}
+	}
+	g_free (fname);
 }
 
 static void
@@ -329,6 +355,8 @@ workbook_do_destroy (Workbook *wb)
 
 	expr_name_clean_workbook (wb);
 
+	gtk_object_destroy (GTK_OBJECT (wb->priv->gui_context));
+	
 	workbook_do_destroy_private (wb);
 
 	if (!GTK_OBJECT_DESTROYED (wb->toplevel))
@@ -390,25 +418,18 @@ workbook_mark_clean (Workbook *wb)
 	g_hash_table_foreach (wb->sheets, cb_sheet_mark_dirty, GINT_TO_POINTER (0));
 }
 
-typedef enum {
-	CLOSE_DENY,
-	CLOSE_ALLOW,
-	CLOSE_RECHECK
-} CloseAction;
-
 static void
 cb_sheet_check_dirty (gpointer key, gpointer value, gpointer user_data)
 {
-	Sheet *sheet = value;
-	GtkWidget *d, *l;
-	int *allow_close = user_data;
-	int button;
-	char *s;
+	Sheet    *sheet = value;
+	gboolean *dirty = user_data;
 
-	if (*allow_close != CLOSE_ALLOW)
+	if (*dirty)
 		return;
 
-	if (!sheet->modified) {
+	if (!sheet->modified)
+		return;
+/*	{
 		GtkEntry   *entry;
 		const char *txt;
 		Cell       *cell;
@@ -430,73 +451,137 @@ cb_sheet_check_dirty (gpointer key, gpointer value, gpointer user_data)
 			if (same)
 				return;
 		}
-	}
+		return;
+		}*/
 
-	d = gnome_dialog_new (
-		_("Warning"),
-		GNOME_STOCK_BUTTON_YES,
-		GNOME_STOCK_BUTTON_NO,
-		GNOME_STOCK_BUTTON_CANCEL,
-		NULL);
-	gnome_dialog_set_parent (GNOME_DIALOG (d), GTK_WINDOW (sheet->workbook->toplevel));
+	*dirty = TRUE;
+}
 
-	if (sheet->workbook->filename)
-		s = g_strdup_printf (
-			_("Workbook %s has unsaved changes, save them?"),
-			g_basename (sheet->workbook->filename));
-	else
-		s = g_strdup (_("Workbook has unsaved changes, save them?"));
+static inline gboolean
+workbook_is_dirty (Workbook *wb)
+{
+	gboolean dirty = FALSE;
+	
+	g_return_val_if_fail (wb != NULL, FALSE);
 
-	l = gtk_label_new (s);
-	gtk_widget_show (l);
-	g_free (s);
+	g_hash_table_foreach (wb->sheets, cb_sheet_check_dirty,
+			      &dirty);
 
-	gtk_box_pack_start (GTK_BOX (GNOME_DIALOG(d)->vbox), l, TRUE, TRUE, 0);
+	return dirty;
+}
 
-	gtk_window_set_position (GTK_WINDOW (d), GTK_WIN_POS_MOUSE);
-	button = gnome_dialog_run_and_close (GNOME_DIALOG (d));
+static void
+cb_sheet_check_pristine (gpointer key, gpointer value, gpointer user_data)
+{
+	Sheet    *sheet = value;
+	gboolean *pristine = user_data;
 
-	switch (button){
-		/* YES */
-	case 0:
-		workbook_save (sheet->workbook);
-		*allow_close = CLOSE_RECHECK;
-		break;
+	if (!sheet_is_pristine (sheet))
+		*pristine = FALSE;
+}
 
-		/* NO */
-	case 1:
-		workbook_mark_clean (sheet->workbook);
-		break;
+/**
+ * workbook_is_pristine:
+ * @wb: 
+ * 
+ *   This checks to see if the workbook has ever been
+ * used ( approximately )
+ * 
+ * Return value: TRUE if we can discard this workbook.
+ **/
+gboolean
+workbook_is_pristine (Workbook *wb)
+{
+	gboolean pristine = TRUE;
 
-		/* CANCEL */
-	case -1:
-	case 2:
-		*allow_close = CLOSE_DENY;
-		break;
+	g_return_val_if_fail (wb != NULL, FALSE);
 
-	}
+	if (workbook_is_dirty (wb))
+		return FALSE;
+
+	if (wb->names || wb->formula_cell_list ||
+#ifdef ENABLE_BONOBO
+	    wb->bonobo_regions ||
+#endif
+	    wb->eval_queue || !wb->needs_name)
+		return FALSE;
+
+	/* Check if we seem to contain anything */
+	g_hash_table_foreach (wb->sheets, cb_sheet_check_pristine,
+			      &pristine);
+
+	return pristine;
 }
 
 static int
 workbook_can_close (Workbook *wb)
 {
-	CloseAction allow_close;
+	gboolean   can_close = TRUE;
+	gboolean   done      = FALSE;
+	int        iteration = 0;
 	static int in_can_close;
 
 	if (in_can_close)
 		return FALSE;
 	
 	in_can_close = TRUE;
-	do {
-		allow_close = CLOSE_ALLOW;
-		g_hash_table_foreach (
-			wb->sheets, cb_sheet_check_dirty, &allow_close);
-	} while (allow_close == CLOSE_RECHECK);
+	while (workbook_is_dirty (wb) &&
+	       !done) {
+		GtkWidget *d, *l;
+		int button;
+		char *s;
 
+		iteration++;
+		
+		d = gnome_dialog_new (
+			_("Warning"),
+			GNOME_STOCK_BUTTON_YES,
+			GNOME_STOCK_BUTTON_NO,
+			GNOME_STOCK_BUTTON_CANCEL,
+			NULL);
+		gnome_dialog_set_parent (GNOME_DIALOG (d), GTK_WINDOW (wb->toplevel));
+		
+		if (wb->filename)
+			s = g_strdup_printf (
+				_("Workbook %s has unsaved changes, save them?"),
+				g_basename (wb->filename));
+		else
+			s = g_strdup (_("Workbook has unsaved changes, save them?"));
+		
+		l = gtk_label_new (s);
+		gtk_widget_show (l);
+		g_free (s);
+		
+		gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (d)->vbox), l, TRUE, TRUE, 0);
+		
+		gtk_window_set_position (GTK_WINDOW (d), GTK_WIN_POS_MOUSE);
+		button = gnome_dialog_run_and_close (GNOME_DIALOG (d));
+		
+		switch (button) {
+			/* YES */
+		case 0:
+			done = workbook_save (wb);
+			break;
+			
+			/* NO */
+		case 1:
+			can_close = TRUE;
+			done      = TRUE;
+			workbook_mark_clean (wb);
+			break;
+			
+			/* CANCEL */
+		case -1:
+		case 2:
+			can_close = FALSE;
+			done      = TRUE;
+			break;
+		}
+	}
+	
 	in_can_close = FALSE;
 	
-	g_assert (allow_close == CLOSE_ALLOW || allow_close == CLOSE_DENY);
-	return allow_close == CLOSE_ALLOW;
+	return can_close;
 }
 
 static void
@@ -563,7 +648,7 @@ copy_cmd (GtkWidget *widget, Workbook *wb)
 	Sheet *sheet;
 
 	sheet = workbook_get_current_sheet (wb);
-	sheet_selection_copy (sheet);
+	sheet_selection_copy (workbook_command_context_gui (wb), sheet);
 }
 
 static void
@@ -573,7 +658,7 @@ cut_cmd (GtkWidget *widget, Workbook *wb)
 
 	sheet = workbook_get_current_sheet (wb);
 	if (sheet->mode == SHEET_MODE_SHEET)
-		sheet_selection_cut (sheet);
+		sheet_selection_cut (workbook_command_context_gui (wb), sheet);
 	else {
 		if (sheet->current_object){
 			gtk_object_unref (GTK_OBJECT (sheet->current_object));
@@ -593,7 +678,8 @@ paste_cmd (GtkWidget *widget, Workbook *wb)
 	g_return_if_fail (!application_clipboard_is_empty ());
 
 	sheet = workbook_get_current_sheet (wb);
-	sheet_selection_paste (sheet, sheet->cursor_col, sheet->cursor_row,
+	sheet_selection_paste (workbook_command_context_gui (wb), sheet,
+			       sheet->cursor_col, sheet->cursor_row,
 			       PASTE_DEFAULT, GDK_CURRENT_TIME);
 }
 
@@ -609,7 +695,8 @@ paste_special_cmd (GtkWidget *widget, Workbook *wb)
 	sheet = workbook_get_current_sheet (wb);
 	flags = dialog_paste_special (wb);
 	if (flags != 0)
-		sheet_selection_paste (sheet, sheet->cursor_col, sheet->cursor_row,
+		sheet_selection_paste (workbook_command_context_gui (wb), sheet,
+				       sheet->cursor_col, sheet->cursor_row,
 				       flags, GDK_CURRENT_TIME);
 
 }
@@ -673,17 +760,22 @@ insert_cols_cmd (GtkWidget *unused, Workbook *wb)
 	Sheet *sheet;
 	int cols;
 
-	/* FIXME : No need to check simplicty.  XL applies for each
+	/* TODO : No need to check simplicty.  XL applies for each
 	 * non-discrete selected region, (use selection_apply) */
 	sheet = workbook_get_current_sheet (wb);
-	if (!selection_is_simple (sheet, _("Insert cols")))
+	if (!selection_is_simple (workbook_command_context_gui (wb), sheet, _("Insert cols")))
 		return;
 
 	ss = sheet->selections->data;
 
-	/* FIXME FIXME : Have we have selected rows rather than columns */
+	/* TODO : Have we have selected rows rather than columns
+	 * This menu item should be disabled when a full row is selected
+	 *
+	 * at minimum a warning if things are about to be cleared ?
+	 */
 	cols = ss->user.end.col - ss->user.start.col + 1;
-	sheet_insert_col (sheet, ss->user.start.col, cols);
+	sheet_insert_cols (workbook_command_context_gui (wb), sheet,
+			   ss->user.start.col, cols);
 }
 
 static void
@@ -693,17 +785,22 @@ insert_rows_cmd (GtkWidget *unused, Workbook *wb)
 	Sheet *sheet;
 	int rows;
 
-	/* FIXME : No need to check simplicty.  XL applies for each
+	/* TODO : No need to check simplicty.  XL applies for each
 	 * non-discrete selected region, (use selection_apply) */
 	sheet = workbook_get_current_sheet (wb);
-	if (!selection_is_simple (sheet, _("Insert rows")))
+	if (!selection_is_simple (workbook_command_context_gui (wb), sheet, _("Insert rows")))
 		return;
 
 	ss = sheet->selections->data;
 
-	/* FIXME FIXME : Have we have selected columns rather than rows */
+	/* TODO : Have we have selected columns rather than rows
+	 * This menu item should be disabled when a full column is selected
+	 *
+	 * at minimum a warning if things are about to be cleared ?
+	 */
 	rows = ss->user.end.row - ss->user.start.row + 1;
-	sheet_insert_row (sheet, ss->user.start.row, rows);
+	sheet_insert_rows (workbook_command_context_gui (wb), sheet,
+			   ss->user.start.row, rows);
 }
 
 static void
@@ -712,7 +809,8 @@ clear_all_cmd (GtkWidget *widget, Workbook *wb)
 	Sheet *sheet;
 
 	sheet = workbook_get_current_sheet (wb);
-	sheet_selection_clear (sheet);
+
+	sheet_selection_clear (workbook_command_context_gui (wb), sheet);
 }
 
 static void
@@ -721,7 +819,7 @@ clear_formats_cmd (GtkWidget *widget, Workbook *wb)
 	Sheet *sheet;
 
 	sheet = workbook_get_current_sheet (wb);
-	sheet_selection_clear_formats (sheet);
+	sheet_selection_clear_formats (workbook_command_context_gui (wb), sheet);
 }
 
 static void
@@ -730,7 +828,7 @@ clear_comments_cmd (GtkWidget *widget, Workbook *wb)
 	Sheet *sheet;
 
 	sheet = workbook_get_current_sheet (wb);
-	sheet_selection_clear_comments (sheet);
+	sheet_selection_clear_comments (workbook_command_context_gui (wb), sheet);
 }
 
 static void
@@ -739,7 +837,7 @@ clear_content_cmd (GtkWidget *widget, Workbook *wb)
 	Sheet *sheet;
 
 	sheet = workbook_get_current_sheet (wb);
-	sheet_selection_clear_content (sheet);
+	sheet_selection_clear_content (workbook_command_context_gui (wb), sheet);
 }
 
 static void
@@ -842,7 +940,6 @@ goal_seek_cmd (GtkWidget *widget, Workbook *wb)
 	dialog_goal_seek (wb, sheet);
 }
 
-#if 0
 static void
 solver_cmd (GtkWidget *widget, Workbook *wb)
 {
@@ -851,7 +948,6 @@ solver_cmd (GtkWidget *widget, Workbook *wb)
 	sheet = workbook_get_current_sheet (wb);
 	dialog_solver (wb, sheet);
 }
-#endif
 
 static void
 data_analysis_cmd (GtkWidget *widget, Workbook *wb)
@@ -958,6 +1054,9 @@ static GnomeUIInfo workbook_menu_edit_clear [] = {
 	GNOMEUIINFO_END
 };
 
+#define PASTE_SPECIAL_NAME N_("P_aste special...")
+#define GNOME_MENU_EDIT_PATH D_("_Edit/")
+
 static GnomeUIInfo workbook_menu_edit [] = {
 	GNOMEUIINFO_MENU_UNDO_ITEM(undo_cmd, NULL),
 	GNOMEUIINFO_MENU_REDO_ITEM(redo_cmd, NULL),
@@ -967,7 +1066,7 @@ static GnomeUIInfo workbook_menu_edit [] = {
         GNOMEUIINFO_MENU_CUT_ITEM(cut_cmd, NULL),
 	GNOMEUIINFO_MENU_COPY_ITEM(copy_cmd, NULL),
 	GNOMEUIINFO_MENU_PASTE_ITEM(paste_cmd, NULL),
-	{ GNOME_APP_UI_ITEM, N_("P_aste special..."), NULL,
+	{ GNOME_APP_UI_ITEM, PASTE_SPECIAL_NAME, NULL,
 	  paste_special_cmd },
         { GNOME_APP_UI_ITEM, N_("_Delete..."), NULL,
 	  delete_cells_cmd },
@@ -1081,9 +1180,7 @@ static GnomeUIInfo workbook_menu_tools [] = {
 	{ GNOME_APP_UI_ITEM, N_("_Sort..."),
 	  N_("Sort the selected cells"), sort_cells_cmd },
 	{ GNOME_APP_UI_ITEM, N_("_Goal Seek..."), NULL, goal_seek_cmd },
-#if 0
 	{ GNOME_APP_UI_ITEM, N_("_Solver..."),    NULL, solver_cmd },
-#endif
 	{ GNOME_APP_UI_ITEM, N_("_Data Analysis..."), NULL, data_analysis_cmd },
 	GNOMEUIINFO_END
 };
@@ -1288,6 +1385,11 @@ wb_edit_key_pressed (GtkEntry *entry, GdkEventKey *event, Workbook *wb)
 
 	case GDK_F4:
 	{
+		/* FIXME FIXME FIXME
+		 * 1) Make sure that a cursor move after an F4
+		 *    does not insert.
+		 * 2) Handle calls for a range rather than a single cell.
+		 */
 		int end_pos = GTK_EDITABLE (entry)->current_pos;
 		int start_pos;
 		int row_status_pos, col_status_pos;
@@ -1363,9 +1465,8 @@ wb_edit_key_pressed (GtkEntry *entry, GdkEventKey *event, Workbook *wb)
 			}
 		}
 
-		/* Select the current range */
-		gtk_editable_select_region (GTK_EDITABLE (entry), start_pos+1, end_pos);
-		GTK_EDITABLE (entry)->current_pos = start_pos+1;
+		/* Do not select the current range, and do not change the position. */
+		gtk_entry_set_position (entry, end_pos);
 	}
 
 	case GDK_KP_Up:
@@ -1842,7 +1943,7 @@ workbook_container_get_object (GnomeObject *container, CORBA_char *item_name,
 		return CORBA_OBJECT_NIL;
 
 	p = strchr (item_name, '!');
-	if (p){
+	if (p) {
 		*p++ = 0;
 
 		if (!range_parse (sheet, p, &range))
@@ -1868,7 +1969,7 @@ workbook_container_get_object (GnomeObject *container, CORBA_char *item_name,
 	/*
 	 * Do we have further configuration information?
 	 */
-	if (range){
+	if (range) {
 		CellRef *a = &range->v.cell_range.cell_a;
 		CellRef *b = &range->v.cell_range.cell_b;
 
@@ -1908,7 +2009,6 @@ workbook_bonobo_setup (Workbook *wb)
 {
 	wb->gnome_container = GNOME_CONTAINER (gnome_container_new ());
 	wb->persist_file = gnome_persist_file_new (
-		GNUMERIC_WORKBOOK_GOAD_ID,
 		workbook_persist_file_load,
 		workbook_persist_file_save,
 		wb);
@@ -2061,6 +2161,8 @@ workbook_new (void)
 	workbook_setup_edit_area (wb);
 	workbook_setup_sheets (wb);
 	gnome_app_set_contents (GNOME_APP (wb->toplevel), wb->table);
+
+	wb->priv->gui_context = command_context_gui_new (wb);
 	
 #ifndef ENABLE_BONOBO
 	gnome_app_create_menus_with_data (GNOME_APP (wb->toplevel), workbook_menu, wb);
@@ -2073,34 +2175,28 @@ workbook_new (void)
 	wb->menu_item_redo		= workbook_menu_edit[1].widget;
 	wb->menu_item_paste		= workbook_menu_edit[5].widget;
 	wb->menu_item_paste_special	= workbook_menu_edit[6].widget;
-
-	/* Disable undo/redo for now */
-	gtk_widget_set_sensitive (wb->menu_item_undo, FALSE);
-	gtk_widget_set_sensitive (wb->menu_item_redo, FALSE);
-
-	/* Disable paste & paste special, they will be enabled when
-	 * there is something to paste */
-	gtk_widget_set_sensitive (wb->menu_item_paste, FALSE);
-	gtk_widget_set_sensitive (wb->menu_item_paste_special, FALSE);
-
 #else
 	{
 		GnomeUIHandlerMenuItem *list;
-		
+
+		wb->bonobo_regions  = NULL;
+		wb->persist_file    = NULL;
 		wb->uih = gnome_ui_handler_new ();
 		gnome_ui_handler_set_app (wb->uih, GNOME_APP (wb->toplevel));
 		gnome_ui_handler_create_menubar (wb->uih);
 		list = gnome_ui_handler_menu_parse_uiinfo_list_with_data (workbook_menu, wb);
 		gnome_ui_handler_menu_add_list (wb->uih, "/", list);
 		gnome_ui_handler_menu_free_list (list);
-
-		wb->menu_item_undo		=
-		wb->menu_item_redo		=
-		wb->menu_item_paste		=
-		wb->menu_item_paste_special	= NULL;
 	}
 #endif
 
+	/* Disable undo/redo for now */
+	workbook_view_set_undo_redo_state (wb, FALSE, FALSE);
+
+	/* Disable paste & paste special, they will be enabled when
+	 * there is something to paste */
+	workbook_view_set_paste_state (wb, 0);
+
  	workbook_create_toolbars (wb);
 
 	/* Minimized pixmap */
@@ -2788,7 +2884,7 @@ workbook_sheets (Workbook *wb)
 	notebook = GTK_NOTEBOOK (wb->notebook);
 	sheets = workbook_sheet_count (wb);
 
-	for (i = 0; i < sheets; i++){
+	for (i = 0; i < sheets; i++) {
 		Sheet *this_sheet;
 		GtkWidget *w;
 
@@ -2853,3 +2949,18 @@ workbook_selection_to_string (Workbook *wb, Sheet *base_sheet)
 	return result;
 }
 
+CommandContext *
+workbook_command_context_gui (Workbook *wb)
+{
+	/* When we are operating before a workbook is created
+	 * wb can be NULL
+	 */
+	if (wb == NULL) {
+		static CommandContext *cc = NULL;
+		if (cc == NULL)
+			cc = command_context_gui_new (NULL);
+		return cc;
+	}
+
+	return wb->priv->gui_context;
+}
