@@ -7,6 +7,7 @@
 #   include <gtk/gtkobject.h>
 #endif
 #include <gtk/gtkwidget.h>
+#include <gnome.h>
 
 #define GNUMERIC_WORKBOOK_GOAD_ID         "IDL:GNOME:Gnumeric:Workbook:1.0"
 #define GNUMERIC_WORKBOOK_FACTORY_GOAD_ID "IDL:GNOME:Gnumeric:WorkbookFactory:1.0"
@@ -25,16 +26,20 @@
 #   define WORKBOOK_PARENT_CLASS_TYPE gtk_object_get_type()
 #endif
 
-/* Forward declaration */
-struct _PrintInformation;
-typedef struct _PrintInformation PrintInformation;
+/*
+ * FIXME FIXME FIXME
+ * WARNING WARNING WARNING
+ * Inorder for this file to work config.h MUST
+ * be included first.
+ *
+ * This seems a poor choice (IMHO).
+ * It would be better for this file to include it directly.
+ */
+#include "gnumeric.h"
+#include "symbol.h"
+#include "summary.h"
 
-struct _WorkbookPrivate;
 typedef struct _WorkbookPrivate WorkbookPrivate;
-
-/* This must be included after the fwd declarations */
-#include "expr.h"
-
 struct _Workbook {
 #ifdef ENABLE_BONOBO
 	/* The base object for the Workbook */
@@ -51,18 +56,18 @@ struct _Workbook {
 	GnomeUIHandler *uih;
 #else
 	GtkObject  gtk_object;
-#endif
-
-	/* { Start view specific elements */;
-        GtkWidget  *toplevel; 
-	GtkWidget  *notebook;
-	GtkWidget  *table;
 
 	/* Menu items that get toggled */
 	GtkWidget  *menu_item_undo;
 	GtkWidget  *menu_item_redo;
 	GtkWidget  *menu_item_paste;
 	GtkWidget  *menu_item_paste_special;
+#endif
+
+	/* { Start view specific elements */;
+        GtkWidget  *toplevel; 
+	GtkWidget  *notebook;
+	GtkWidget  *table;
 
 	/* Edit area */
 	GtkWidget  *ea_status;
@@ -129,11 +134,11 @@ Workbook   *workbook_new_with_sheets     (int sheet_count);
 
 gboolean    workbook_set_filename        (Workbook *, const char *);
 void        workbook_set_title           (Workbook *, const char *);
-Workbook   *workbook_try_read            (const char *filename);
+Workbook   *workbook_try_read            (const char *filename, char **error_msg);
 Workbook   *workbook_read                (const char *filename);
 
-void        workbook_save_as             (Workbook *);
-void        workbook_save                (Workbook *);
+gboolean    workbook_save_as             (Workbook *);
+gboolean    workbook_save                (Workbook *);
 void        workbook_print               (Workbook *, gboolean);
 void        workbook_attach_sheet        (Workbook *, Sheet *);
 gboolean    workbook_detach_sheet        (Workbook *, Sheet *, gboolean);
@@ -148,6 +153,7 @@ int         workbook_parse_and_jump      (Workbook *wb, const char *text);
 Sheet      *workbook_sheet_lookup        (Workbook *wb, const char *sheet_name);
 void        workbook_mark_clean          (Workbook *wb);
 void        workbook_set_dirty           (Workbook *wb, gboolean is_dirty);
+gboolean    workbook_is_pristine         (Workbook *wb);
 gboolean    workbook_rename_sheet        (Workbook *wb,
 					  const char *old_name,
 					  const char *new_name);
@@ -176,6 +182,8 @@ typedef gboolean (*WorkbookCallback)(Workbook *, gpointer data);
 void     workbook_foreach             (WorkbookCallback cback,
 				       gpointer data);
 
+CommandContext *workbook_command_context_gui (Workbook *wb);
+
 /*
  * Feedback routines
  */
